# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com
# Author: matt.clark@eucalyptus.com
from boto.ec2.image import Image
from boto.ec2.volume import Volume
from cwops import CWops
from asops import ASops
from eucaops.elbops import ELBops
from iamops import IAMops
from ec2ops import EC2ops
from s3ops import S3ops
from stsops import STSops
import time
from eutester.euservice import EuserviceManager
from boto.ec2.instance import Reservation
from boto.exception import EC2ResponseError
from eutester.euconfig import EuConfig
from eutester.euproperties import Euproperty_Manager
from eutester.machine import Machine
from eutester.euvolume import EuVolume
from eutester import eulogger
import re
import os

class Eucaops(EC2ops,S3ops,IAMops,STSops,CWops, ASops, ELBops):
    
    def __init__(self, config_file=None, password=None, keypath=None, credpath=None, aws_access_key_id=None,
                 aws_secret_access_key = None,  account="eucalyptus", user="admin", username=None, APIVersion='2011-01-01',
                 region=None, ec2_ip=None, s3_ip=None, as_ip=None, elb_ip=None, download_creds=True,boto_debug=0,
                 debug_method=None):
        self.config_file = config_file 
        self.APIVersion = APIVersion
        self.eucapath = "/opt/eucalyptus"
        self.ssh = None
        self.sftp = None
        self.clc = None
        self.password = password
        self.keypath = keypath
        self.timeout = 30
        self.delay = 0
        self.exit_on_fail = 0
        self.fail_count = 0
        self.start_time = time.time()
        self.key_dir = "./"
        self.clc_index = 0
        self.credpath = credpath
        self.download_creds = download_creds
        self.logger = eulogger.Eulogger(identifier="EUCAOPS")
        self.debug = debug_method or self.logger.log.debug
        self.critical = self.logger.log.critical
        self.info = self.logger.log.info
        self.username = username
        self.account_id = None
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key


        if self.config_file is not None:
            ## read in the config file
            self.debug("Reading config file: " + config_file)
            self.config = self.read_config(config_file)

            ### Set the eucapath
            try:
                if "REPO" in self.config["machines"][0].source:
                    self.eucapath="/"
            except Exception, e:
                raise Exception("Could not get REPO info from input file\n" + str(e))

            ### No credpath but does have password and an ssh connection to the CLC
            ### Private cloud with root access 
            ### Need to get credentials for the user if there arent any passed in
            ### Need to create service manager for user if we have an ssh connection and password
            clc_array = self.get_component_machines("clc")
            self.clc = clc_array[0]
            walrus_array = self.get_component_machines("ws")
            self.walrus = walrus_array[0]
            self.sftp = self.clc.ssh.connection.open_sftp()
            if self.download_creds:
                if self.credpath is None:
                    ### TRY TO GET CREDS ON FIRST CLC if it fails try on second listed clc, if that fails weve hit a terminal condition
                    try:
                        self.debug("Attempting to get credentials and setup sftp")
                        self.sftp = self.clc.ssh.connection.open_sftp()
                        self.credpath = self.get_credentials(account,user)
                        self.debug("Successfully downloaded and synced credentials")
                    except Exception, e:
                        tb = self.get_traceback()
                        self.debug("Caught an exception when getting credentials from first CLC: " + str(e))
                        ### If i only have one clc this is a critical failure, else try on the other clc
                        if len(clc_array) < 2:
                            raise Exception(str(tb) + "\nCould not get credentials from first CLC and no other to try")
                        self.swap_clc()
                        self.sftp = self.clc.ssh.connection.open_sftp()
                        self.get_credentials(account,user)
                        
                self.service_manager = EuserviceManager(self)
                self.clc = self.service_manager.get_enabled_clc().machine
                self.walrus = self.service_manager.get_enabled_walrus().machine 



        if self.credpath and not aws_access_key_id:
            aws_access_key_id = self.get_access_key()
        if self.credpath and not aws_secret_access_key:
            aws_secret_access_key = self.get_secret_key()
        self.test_resources = {}
        if self.download_creds:
            try:
                if self.credpath and not ec2_ip:
                    ec2_ip = self.get_ec2_ip()
                self.setup_ec2_connection(endpoint=ec2_ip, path="/services/Eucalyptus", port=8773, is_secure=False, region=region, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key, APIVersion=APIVersion, boto_debug=boto_debug)
                self.setup_ec2_resource_trackers()
                self.setup_iam_connection(endpoint=ec2_ip, path="/services/Euare", port=8773, is_secure=False, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key,  boto_debug=boto_debug)
                self.setup_sts_connection( endpoint=ec2_ip, path="/services/Eucalyptus", port=8773, is_secure=False, region=region, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
                self.setup_cw_connection( endpoint=ec2_ip, path="/services/CloudWatch", port=8773, is_secure=False, region=region, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
                self.setup_cw_resource_trackers()
            except Exception, e:
                tb = self.get_traceback()
                raise Exception(tb + "\nUnable to create EC2 connection because of: " + str(e) )

            try:
                if self.credpath and not s3_ip:
                    s3_ip = self.get_s3_ip()
                self.setup_s3_connection(endpoint=s3_ip, path="/services/Walrus", port=8773, is_secure=False,aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key,  boto_debug=boto_debug)
                self.setup_s3_resource_trackers()
            except Exception, e:
                raise Exception("Unable to create S3 connection because of: " + str(e) )

            try:
                if self.credpath and not as_ip:
                    as_ip = self.get_as_ip()
                self.setup_as_connection(endpoint=as_ip, path="/services/AutoScaling", port=8773, is_secure=False, region=region, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
            except Exception, e:
                self.debug("Unable to create AS connection because of: " + str(e) )

            try:
                if self.credpath and not elb_ip:
                    elb_ip = self.get_elb_ip()
                self.setup_elb_connection(endpoint=elb_ip, path="/services/LoadBalancing", port=8773, is_secure=False, region=region, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key, boto_debug=boto_debug)
            except Exception, e:
                self.debug("Unable to create ELB connection because of: " + str(e) )
        if self.clc and account == 'eucalytpus':
            self.update_property_manager()

    def get_available_vms(self, type=None, zone=None):
        """
        Get available VMs of a certain type or return a dictionary with all types and their available vms
        type        VM type to get available vms 
        """
        
        zones = self.ec2.get_all_zones("verbose")
        if type is None:
            type = "m1.small"
        ### Look for the right place to start parsing the zones
        zone_index = 0
        if zone is not None:
            while zone_index < len(zones):
                current_zone = zones[zone_index]
                if re.search( zone, current_zone.name):
                    break
                zone_index += 20
            if zone_index > (len(zones) - 1):
                raise Exception("Unable to find Availability Zone")    
        else:
            zone = zones[0].name
            
        ### Inline switch statement
        type_index = {  "t1.micro": 2,
                        "m1.small": 3,
                        "m1.large": 4,
                        "m1.xlarge" : 5,
                        "c1.xlarge" : 6,
                        "m2.xlarge" : 7,
                        "c1.medium" : 8,
                        "m1.medium" : 9,
                        "m3.xlarge" : 10,
                        "m2.2xlarge" : 11,
                        "m3.2xlarge" : 12,
                        "m2.4xlarge" : 13,
                        "cc1.4xlarge" : 14,
                        "hi1.4xlarge" : 15,
                        "cc2.8xlarge" : 16,
                        "cg1.4xlarge" : 17,
                        "cr1.8xlarge" : 18,
                        "hs1.8xlarge" : 19
                      }[type] 
        type_state = zones[ zone_index + type_index ].state.split()
        self.debug("Finding available VMs: Partition=" + zone +" Type= " + type + " Number=" +  str(int(type_state[0])) )
        return int(type_state[0])
        
    
    
            
    def modify_property(self, property, value):
        """
        Modify a eucalyptus property through the command line euca-modify-property tool
        property        Property to modify
        value           Value to set it too
        """
        command = "source " + self.credpath + "/eucarc && " + self.eucapath + "/usr/sbin/euca-modify-property -p " + property + "=" + value
        if self.clc.found(command, property):
            self.debug("Properly modified property " + property)
        else:
            raise Exception("Setting property " + property + " failed")
    
   
    def cleanup_artifacts(self,instances=True, snapshots=True, volumes=True, load_balancers=True):
        """
        Description: Attempts to remove artifacts created during and through this eutester's lifespan.
        """
        failmsg = ""
        failcount = 0
        self.debug("Starting cleanup of artifacts")
        if instances:
            for res in self.test_resources["reservations"]:
                try:
                    self.terminate_instances(res)
                except Exception, e:
                    tb = self.get_traceback()
                    failcount +=1
                    failmsg += str(tb) + "\nError#:"+ str(failcount)+ ":" + str(e)+"\n"
        if volumes:
            try:
                self.clean_up_test_volumes(timeout_per_vol=60)
            except Exception, e:
                tb = self.get_traceback()
                failcount +=1
                failmsg += str(tb) + "\nError#:"+ str(failcount)+ ":" + str(e)+"\n"
        if snapshots:
            try:
                self.cleanup_test_snapshots()
            except Exception, e:
                tb = self.get_traceback()
                failcount +=1
                failmsg += str(tb) + "\nError#:"+ str(failcount)+ ":" + str(e)+"\n"
        if load_balancers:
            try:
                self.cleanup_load_balancers()
            except Exception, e:
                tb = self.get_traceback()
                failcount +=1
                failmsg += str(tb) + "\nError#:"+ str(failcount)+ ":" + str(e)+"\n"

        for key,array in self.test_resources.iteritems():
            for item in array:
                try:
                    ### SWITCH statement for particulars of removing a certain type of resources
                    self.debug("Deleting " + str(item))
                    if isinstance(item, Image):
                        item.deregister()
                    elif isinstance(item, Reservation):
                        self.terminate_instances(item)
                    elif isinstance(item, Volume):
                        try:
                            self.detach_volume(item)
                        except:
                            pass
                        item.update()
                        if item.status != 'deleted':
                            self.delete_volume(item)
                    else:
                        item.delete()
                except Exception, e:
                    tb = self.get_traceback()
                    failcount += 1
                    failmsg += str(tb) + "\nUnable to delete item: " + str(item) + "\n" + str(e)+"\n"
        if failmsg:
            failmsg += "\nFound " + str(failcount) + " number of errors while cleaning up. See above"
            raise Exception(failmsg)

    def cleanup_load_balancers(self, lbs=None):
        """
        :param lbs: optional list of load balancers, otherwise it will attempt to delete from test_resources[]
        """
        if lbs:
            self.delete_load_balancers(lbs)
        else:
            try:
                self.delete_load_balancers(self.test_resources['load_balancers'])
            except KeyError:
                self.debug("No loadbalancers to delete")

    def cleanup_test_snapshots(self,snaps=None, clean_images=False, add_time_per_snap=10, wait_for_valid_state=120, base_timeout=180):
        """
        :param snaps: optional list of snapshots, else will attempt to delete from test_resources[]
        :param clean_images: Boolean, if set will attempt to delete registered images referencing the snapshots first.
                             Images referencing the snapshot may prevent snapshot deletion to protect the image.
        :param add_time_per_snap:  int number of seconds to append to base_timeout per snapshot
        :param wait_for_valid_state: int seconds to wait for snapshot(s) to enter a 'deletable' state
        :param base_timeout: base timeout to use before giving up, and failing operation.
        """
        snaps = snaps or self.test_resources['snapshots']
        if not snaps:
            return
        self.debug('Attempting to clean the following snapshots:')
        self.print_eusnapshot_list(snaps)
        if clean_images:
            for snap in snaps:
                for image in self.test_resources['images']:
                    for dev in image.block_device_mapping:
                        if image.block_device_mapping[dev].snapshot_id == snap.id:
                            self.delete_image(image)
        if snaps:
            return self.delete_snapshots(snaps,
                                        base_timeout=base_timeout,
                                        add_time_per_snap=add_time_per_snap,
                                        wait_for_valid_state=wait_for_valid_state)




    def clean_up_test_volumes(self, volumes=None, min_timeout=180, timeout_per_vol=30):
        """
        Definition: cleaup helper method intended to clean up volumes created within a test, after the test has ran.

        :param volumes: optional list of volumes to delete from system, otherwise will use test_resources['volumes']
        """
        euvolumes = []
        detaching = []
        not_exist = []
        line = '\n----------------------------------------------------------------------------------------------------\n'

        volumes = volumes or self.test_resources['volumes']
        if not volumes:
            self.debug('clean_up_test_volumes, no volumes passed to delete')
            return
        self.debug('clean_up_test_volumes starting\nVolumes to be deleted:' + ",".join(str(x) for x in volumes))



        for vol in volumes:
            try:
                vol = self.get_volume(volume_id=vol.id)
            except:
                tb = self.get_traceback()
                self.debug("\n" + line + " Ignoring caught Exception:\n" + str(tb) + "\n"+ str(vol.id) +
                           ', Could not retrieve volume, may no longer exist?' + line)
                vol = None
            if vol:
                try:
                    vol.update()
                    if not isinstance(vol, EuVolume):
                        vol = EuVolume.make_euvol_from_vol(vol, self)
                    euvolumes.append(vol)
                except:
                    tb = self.get_traceback()
                    self.debug('Ignoring caught Exception: \n' + str(tb))
        try:
            self.debug('Attempting to clean up the following volumes:')
            self.print_euvolume_list(euvolumes)
        except: pass
        self.debug('Clean_up_volumes: Detaching any attached volumes to be deleted...')
        for vol in euvolumes:
            try:
                vol.update()
                if vol.status == 'in-use':
                    if vol.attach_data and (vol.attach_data.status != 'detaching' or vol.attach_data.status != 'detached'):
                        try:
                            self.debug(str(vol.id) + ', Sending detach. Status:' +str(vol.status) +
                                       ', attach_data.status:' + str(vol.attach_data.status))
                            vol.detach()
                        except EC2ResponseError, be:
                            if 'Volume does not exist' in be.error_message:
                                not_exist.append(vol)
                                self.debug(str(vol.id) + ', volume no longer exists')
                            else:
                                raise be
                    detaching.append(vol)
            except:
                print self.get_traceback()
        #If the volume was found to no longer exist on the system, remove it from further monitoring...
        for vol in not_exist:
            if vol in detaching:
                detaching.remove(vol)
            if vol in euvolumes:
                euvolumes.remove(vol)
        self.test_resources['volumes'] = euvolumes
        timeout = min_timeout + (len(volumes) * timeout_per_vol)
        #If detaching wait for detaching to transition to detached...
        if detaching:
            self.monitor_euvolumes_to_status(detaching, status='available', attached_status=None,timeout=timeout)
        self.debug('clean_up_volumes: Deleteing volumes now...')
        self.print_euvolume_list(euvolumes)
        if euvolumes:
            self.delete_volumes(euvolumes, timeout=timeout)

                    
    def get_current_resources(self,verbose=False):
        '''Return a dictionary with all known resources the system has. Optional pass the verbose=True flag to print this info to the logs
           Included resources are: addresses, images, instances, key_pairs, security_groups, snapshots, volumes, zones
        '''
        current_artifacts = dict()
        current_artifacts["addresses"] = self.ec2.get_all_addresses()
        current_artifacts["images"] = self.ec2.get_all_images()
        current_artifacts["instances"] = self.ec2.get_all_instances()
        current_artifacts["key_pairs"] = self.ec2.get_all_key_pairs()
        current_artifacts["security_groups"] = self.ec2.get_all_security_groups()
        current_artifacts["snapshots"] = self.ec2.get_all_snapshots()
        current_artifacts["volumes"] = self.ec2.get_all_volumes()
        current_artifacts["zones"] = self.ec2.get_all_zones()
        
        if verbose:
            self.debug("Current resources in the system:\n" + str(current_artifacts))
        return current_artifacts
    
    def read_config(self, filepath, username="root"):
        """ Parses the config file at filepath returns a dictionary with the config
            Config file
            ----------
            The configuration file for (2) private cloud mode has the following structure:
            
                clc.mydomain.com CENTOS 5.7 64 REPO [CC00 CLC SC00 WS]    
                nc1.mydomain.com VMWARE ESX-4.0 64 REPO [NC00]
            
            Columns
            ------ 
                IP or hostname of machine   
                Distro installed on machine - Options are RHEL, CENTOS, UBUNTU additionally VMWARE can be used for NCs 
                Distro version on machine  - RHEL (5.x, 6.x), CentOS (5.x), UBUNTU (LUCID)
                Distro base architecture  - 32 or 64
                System built from packages (REPO) or source (BZR), packages assumes path to eucalyptus is /, bzr assumes path to eucalyptus is /opt/eucalyptus
                List of components installed on this machine encapsulated in brackets []
            
            These components can be:
            
                CLC - Cloud Controller   
                WS - Walrus   
                SC00 - Storage controller for cluster 00   
                CC00 - Cluster controller for cluster 00    
                NC00 - A node controller in cluster 00   
        """
        config_hash = {}
        machines = []
        f = None
        try:
            #f = open(filepath, 'r')
            self.testconf = EuConfig(filepath, legacy_qa_config=True)
            f = self.testconf.legacybuf.splitlines()
        except IOError as (errno, strerror):
            self.debug( "ERROR: Could not find config file " + self.config_file)
            raise
            
        for line in f:
            ### LOOK for the line that is defining a machine description
            line = line.strip()
            re_machine_line = re.compile(".*\[.*]")
            if re_machine_line.match(line):
                machine_details = line.split(None, 5)
                machine_dict = {}
                machine_dict["hostname"] = machine_details[0]
                machine_dict["distro"] = machine_details[1]
                machine_dict["distro_ver"] = machine_details[2]
                machine_dict["arch"] = machine_details[3]
                machine_dict["source"] = machine_details[4]
                machine_dict["components"] = map(str.lower, machine_details[5].strip('[]').split())

                ### ADD the machine to the array of machine
                cloud_machine = Machine(   machine_dict["hostname"], 
                                        distro = machine_dict["distro"], 
                                        distro_ver = machine_dict["distro_ver"], 
                                        arch = machine_dict["arch"], 
                                        source = machine_dict["source"], 
                                        components = machine_dict["components"],
                                        connect = True,
                                        password = self.password,
                                        keypath = self.keypath,
                                        username = username
                                        )
                machines.append(cloud_machine)
                
            ### LOOK for network mode in config file if not found then set it unknown
            try:
                if re.search("^network",line, re.IGNORECASE):
                    config_hash["network"] = line.split()[1].lower()
            except:
                self.debug("Could not find network type setting to unknown")
                config_hash["network"] = "unknown"
        #f.close()   
        config_hash["machines"] = machines 
        return config_hash

    def update_property_manager(self,machine=None):
        machine = machine or self.clc
        self.property_manager = Euproperty_Manager(self,debugmethod=self.debug)

    def swap_clc(self):
        all_clcs = self.get_component_machines("clc")
        if self.clc is all_clcs[0]:
            self.debug("Swapping CLC from " + all_clcs[0].hostname + " to " + all_clcs[1].hostname)
            self.clc = all_clcs[1]
            
        elif self.clc is all_clcs[1]:
            self.debug("Swapping CLC from " + all_clcs[1].hostname + " to " + all_clcs[0].hostname)
            self.clc = all_clcs[0]

    def swap_walrus(self):
        all_walruses = self.get_component_machines("ws")
        if self.walrus is all_walruses[0]: 
            self.debug("Swapping Walrus from " + all_walruses[0].hostname + " to " + all_walruses[1].hostname)
            self.walrus = all_walruses[1]
        elif self.walrus is all_walruses[1]:
            self.debug("Swapping Walrus from " + all_walruses[1].hostname + " to " + all_walruses[0].hostname)
            self.walrus = all_walruses[0]
            
    def get_network_mode(self):
        return self.config['network']
            
    def get_component_ip(self, component):
        """ Parse the machine list and a bm_machine object for a machine that matches the component passed in"""
        #loop through machines looking for this component type
        component.lower()
        machines_with_role = [machine.hostname for machine in self.config['machines'] if component in machine.components]
        if len(machines_with_role) == 0:
            raise Exception("Could not find component "  + component + " in list of machines")
        else:
            return machines_with_role[0]
    
    def get_machine_by_ip(self, hostname):
        machines = [machine for machine in self.config['machines'] if re.search(hostname, machine.hostname)]
        
        if machines is None or len(machines) == 0:
            self.fail("Could not find machine at "  + hostname + " in list of machines")
            return None
        else:
            return machines[0]

    def get_component_machines(self, component = None):
        #loop through machines looking for this component type
        """ Parse the machine list and a list of bm_machine objects that match the component passed in"""
        if component is None:
            return self.config['machines']
        else:
            component.lower()
            machines_with_role = [machine for machine in self.config['machines'] if re.search(component, " ".join(machine.components))]
            if len(machines_with_role) == 0:
                raise IndexError("Could not find component "  + component + " in list of machines")
            else:
                return machines_with_role

    def swap_component_hostname(self, hostname):
        if hostname != None:
            if len(hostname) < 5:
                component_hostname = self.get_component_ip(hostname)
                hostname = component_hostname
        return hostname
       
    def get_credentials(self, account="eucalyptus", user="admin"):
        """Login to the CLC and download credentials programatically for the user and account passed in
           Defaults to admin@eucalyptus 
        """
        self.debug("Starting the process of getting credentials")
        
        ### GET the CLCs from the config file
        clcs = self.get_component_machines("clc")
        if len(clcs) < 1:
            raise Exception("Could not find a CLC in the config file when trying to get credentials")
        
        admin_cred_dir = "eucarc-" + clcs[0].hostname + "-" + account + "-" + user 
        cred_file_name = "creds.zip"
        full_cred_path = admin_cred_dir + "/" + cred_file_name
        
        ### IF I dont already have credentials, download and sync them
        if self.credpath is None:
            ### SETUP directory remotely
            self.setup_remote_creds_dir(admin_cred_dir)
            
            ### Create credential from Active CLC
            self.create_credentials(admin_cred_dir, account, user)
        
            ### SETUP directory locally
            self.setup_local_creds_dir(admin_cred_dir)
          
            ### DOWNLOAD creds from clc
            self.download_creds_from_clc(admin_cred_dir)

            ### SET CREDPATH ONCE WE HAVE DOWNLOADED IT LOCALLY
            self.credpath = admin_cred_dir
            ### IF there are 2 clcs make sure to sync credentials across them
        ### sync the credentials  to all CLCs
        for clc in clcs:
            self.send_creds_to_machine(admin_cred_dir, clc)

        return admin_cred_dir
   
    def create_credentials(self, admin_cred_dir, account, user):
       
        cred_dir =  admin_cred_dir + "/creds.zip"
        self.sys('rm -f '+cred_dir)
        cmd_download_creds = self.eucapath + "/usr/sbin/euca_conf --get-credentials " + admin_cred_dir + "/creds.zip " + "--cred-user "+ user +" --cred-account " + account 
       
        if self.clc.found( cmd_download_creds, "The MySQL server is not responding"):
            raise IOError("Error downloading credentials, looks like CLC was not running")
        if self.clc.found( "unzip -o " + admin_cred_dir + "/creds.zip " + "-d " + admin_cred_dir, "cannot find zipfile directory"):
            raise IOError("Empty ZIP file returned by CLC")
       
        
    
    def download_creds_from_clc(self, admin_cred_dir):
        self.debug("Downloading credentials from " + self.clc.hostname + ", path:" + admin_cred_dir + "/creds.zip")
        self.sftp.get(admin_cred_dir + "/creds.zip" , admin_cred_dir + "/creds.zip")
        os.system("unzip -o " + admin_cred_dir + "/creds.zip -d " + admin_cred_dir )
    
    def send_creds_to_machine(self, admin_cred_dir, machine):
        self.debug("Sending credentials to " + machine.hostname)
        try:
            machine.sftp.listdir(admin_cred_dir + "/creds.zip")
            self.debug("Machine " + machine.hostname + " already has credentials in place")
        except IOError, e:
            machine.sys("mkdir " + admin_cred_dir)
            machine.sftp.put( admin_cred_dir + "/creds.zip" , admin_cred_dir + "/creds.zip")
            machine.sys("unzip -o " + admin_cred_dir + "/creds.zip -d " + admin_cred_dir )
            machine.sys("sed -i 's/" + self.get_ec2_ip() + "/" + machine.hostname  +"/g' " + admin_cred_dir + "/eucarc")
            
        
    def setup_local_creds_dir(self, admin_cred_dir):
        if not os.path.exists(admin_cred_dir):
            os.mkdir(admin_cred_dir)
      
    def setup_remote_creds_dir(self, admin_cred_dir):
        self.sys("mkdir " + admin_cred_dir)
    
    def sys(self, cmd, verbose=True, listformat=True, timeout=120, code=None):
        """ By default will run a command on the CLC machine, the connection used can be changed by passing a different hostname into the constructor
            For example:
            instance = Eutester( hostname=instance.ip_address, keypath="my_key.pem")
            instance.sys("mount") # check mount points on instance and return the output as a list
        """
        return self.clc.sys(cmd, verbose=verbose,  listformat=listformat, timeout=timeout, code=code)
    

    



'''
Created on Nov 10, 2011
@author: mclark
'''
#from eutester import Eutester
from eucaops import Eucaops
from selenium import webdriver
from selenium.webdriver.firefox.firefox_profile import FirefoxProfile
#from selenium.webdriver.common.by import By
from selenium.common.exceptions import NoSuchElementException
import unittest, time, re, os, signal, commands
from subprocess import call
import euwebglobals, euwebaccount, euwebuser
import eulogger
#import string, random
#import sys, inspect



class Eucaweb(unittest.TestCase):   
    def __init__ (self, host="localhost",
                  browser="Firefox", 
                  clc="", 
                  user="admin", 
                  account="eucalyptus", 
                  adminpasswd="",
                  clcpasswd="", 
                  needEucaops=True,
                  downloadPath="/tmp/eucawebdriver",
                  configFile=""):
        
        self.images = []
        self.loggedIn = False
        self.host = host
        self.browser = str(browser).upper()
        self.clc = clc
        self.clcpasswd = clcpasswd
        self.clcurl = "https://"+clc+":8443/"
        self.user = user
        self.account = account
        self.passwd = adminpasswd
        self.defTimeout = 10
        self.configFile = configFile
        self.needEucaops = needEucaops
        self.errs = []
        self.downloadPath = downloadPath
        if (re.search('/$',self.downloadPath) ):
                self.downloadPath = self.downloadPath.rstrip('/')
        self.wg = euwebglobals.Webui_globals()
        self.setUp()
    
        
    def setUp(self):
        try:
            self.logger = eulogger.Eulogger(name='euca')
            self.log = self.logger.log
            self.printSelf()
            self.log.debug("setUp starting...")
            self.errs = []           
            self.startDriver()            
            if  (self.configFile == "" ):
                fpath = str(os.path.abspath( __file__ ))#get the abs path to file
                thisfile = re.search( '\w+.\w+.$', fpath).group(0) #get thisfile so we can remove it to get abs path
                fpath = str(os.path.abspath( __file__ )).replace(thisfile,"")
                self.configFile = str(fpath)+"2b_tested.lst"                 
            self.printSelf()    
           
            
            if (self.needEucaops):
                try:
                    retcode = call('rm -rf eucarc-*',shell=True)
                    print "removed a local eucarc =" + str(retcode)
                except Exception,e:
                    self.log.debug("couldn't delete the local eucarc")
                    pass
                    
                try:
                    self.eutester = Eucaops(hostname="clc", password=self.clcpasswd,config_file=self.configFile)
                    #self.eutester = Eutester(config_file=self.configFile, password=self.clcpasswd)
                except Exception, ute:
                    self.log.critical("failed to create eutester object within gui...!!:"+str(ute))
                    raise ute
            if (self.clc == ""):
                self.clc = self.eutester.get_clc_ip()
                self.clcurl = "https://"+self.clc+":8443/"
            self.driver.implicitly_wait(10)
            self.base_url = self.clcurl
            bname = self.driver.name
            self.log.debug("Found browser name:" + str(bname))
        except Exception, e:
            self.log.critical("Failed setup!!:\n"+ str(e))
            self.tearDown(0)
            raise e
        
    def startDriver(self):
        #dbg = self.log.debug
        try:
            self.log.critical("STarting driver!!!!!")
            if (self.browser == "FIREFOX" ):
                self.log.debug("using webdriver firefox, setting up custom profile...")
                ffprof = FirefoxProfile()
                #ffprof.accept_untrusted_certs("true")
                ffprof.set_preference("browser.download.folderList", 2)
                ffprof.set_preference("browser.download.dir",self.downloadPath)
                ffprof.set_preference("browser.helperApps.neverAsk.saveToDisk", "application/zip") #to download credentials without "save as" dialog
                ffprof.set_preference("browser.download.manager.showWhenStarting", "false")
                self.driver = webdriver.Firefox(ffprof)
            elif (self.browser == "CHROME" ):
                self.log.debug("using webdriver chrome")
                self.driver = webdriver.Chrome()
            elif (self.browser == "IE" ):
                self.log.debug( "using webdriver IE" )
                self.driver = webdriver.Ie()
            else:
                self.driver = webdriver.Firefox()
        except Exception, de:
            self.log.critical("startDriver error:" + str(de))
            raise de
            
                
        
        
    def printSelf(self):
        self.log.debug( "++++++++++++++++++++++++++++++++++++++++++++++")
        self.log.debug( "configFile: " + str(self.configFile))
        self.log.debug( "host: " + str(self.host))
        self.log.debug( "browser: " + str(self.browser))
        self.log.debug( "clc: " + str(self.clc))
        self.log.debug( "clcpasswd:"+str(self.clcpasswd))
        self.log.debug( "user: " + str(self.user))
        self.log.debug( "account: " + str(self.account))
        self.log.debug( "password: " + str(self.passwd))
        self.log.debug( "needEucaops: " + str(self.needEucaops))
        self.log.debug( "xpathUserLink: " + self.wg.xUserLink)
        self.log.debug( "+++++++++++++++++++++++++++++++++++++++++++++++")
  

    def waitForText(self, xpath, text, retry):
        '''
        Will check current xpath for text supplied for a given number of times
        It appears the find_element_by_path also has a buitl in wait period. 
        This might be referencing the webdriver's "driver.implicitly_wait"value
        '''
        self.log.debug(self.hd("waitForText ("+ str(text) +") starting..."))
        driver = self.driver
     
        for i in range(retry):
            self.log.debug("retry count:"+str(i))
            try:
                cmd = xpath + "[contains(.,'" + text + "')]"
                self.log.debug("Looking for\""+text+"\" in xpath: "+ xpath)
                driver.find_element_by_xpath(cmd)
                self.log.debug("Found text\"" + text + "\" after " + str(i) + " retries")
                return 
                break
            except Exception, e:
                self.log.critical("Did nof find text on pass("+str(i)+":" + str(e))
                time.sleep(1)
                pass
        #else:   
        self.log.critical("Failed to find (" + text + ") after (" + str(retry) + ") tries in xpath (" + xpath + ")")
        raise Exception("Failed to find text("+text+")")
            
    def goLoginPage(self):
        self.log.debug(self.hd("goLoginPage"))
        driver = self.driver
        driver.get(self.clcurl)
        try:
            self.waitForTextOnPage("Password")
        except Exception, e:
            self.log.critical("Password text not found, err:"+ str(e))
            
    
        
    def get(self,url):
        return self.driver.get(url)
       
        
    def getElmByText(self, xpath, text):
        '''
        get web element in given xpath that contains the given text
        '''
        self.log.debug(self.hd("getElmByText:"+str(text)))
        driver = self.driver
        cmd = xpath + "[contains(.,'" + text + "')]"
        self.log.debug("Our xpath: "+ xpath) 
        elm = driver.find_element_by_xpath(cmd)
        self.log.debug("Found text(" + text + ")")
        return elm
    
    
    def hd(self,msg):
        '''
        format 'msg' into a debug header  
        '''
        myHdr = "<<<<( "+msg+" )>>>>"
        '''
        myHdr = "\n+++++++++++++++++++++++++++++++++++++++++++++++\n"    
        myHdr = myHdr+"    "+msg+"\n"
        myHdr = myHdr+"+++++++++++++++++++++++++++++++++++++++++++++++\n"
        '''    
        return myHdr
    
    def login(self,account,user,passwd):
        '''
        Log 'user' into 'account' using password 'passwd'
        '''
        account = str(account)
        user = str(user)
        passwd = str(passwd)
        self.log.debug(self.hd("login (account:"+account+" user:"+user+" passwd:"+passwd+")"))
        
        driver = self.driver
        driver.get(self.clcurl)
        driver.find_element_by_id("accountName").clear()
        driver.find_element_by_id("accountName").send_keys(str(account))
        driver.find_element_by_id("userName").clear()
        driver.find_element_by_id("userName").send_keys(str(user))
        driver.find_element_by_id("password").clear()
        driver.find_element_by_id("password").send_keys(str(passwd))
        driver.find_element_by_css_selector("input[type=\"submit\"]").click()
        # We should be logged in at this point,
        # verify by checking the user@account link at top of page...
        #if (self.verifyUserLinkPresent(user,account) == False):
        
        try:
            self.waitForTextOnPage(user+'@'+account, 3 )
            self.log.debug("USER:"+str(user)+" ACCOUNT:"+account+" LOGGED IN")
        except Exception, e:
            self.log.critical("Failed to verify user link for login (account:"+account+" user:" +user+" passwd:"+passwd+")")
            raise e
            
        
            
            
        
        
    #check entire page aka 
    def waitForTextOnPage(self,searchtext, retry=1):
        '''
        check entire page for a certain text string 'searchtext'
        retry 'retry' amount of times
        '''
        self.log.debug(self.hd("waitForTextOnPage starting, text:("+str(searchtext)+")"))
        xpath = '//*' #search whole page
        try:
            self.waitForText(xpath, searchtext, 1)
            self.log.debug("Found Text:"+ str(searchtext))    
        except Exception, e:
            self.log.debug("Failed to find text (" + searchtext + ")on page")
            raise e
        
    
    
    def verifyUserLinkPresent(self,username,accountname):
        '''
        Confirms the user specific link name@account is present on the page
        '''
        try:
            ltext = username+'@'+accountname
            self.log.debug(self.hd("verifyUserLink starting("+ltext+")..."))
            webelm = self.getUserLink()
            self.log.debug("Userlink elm:"+webelm.text)
            if (webelm.text == ltext):
                self.log.debug("Userlinks matched: ltext:"+ltext+" vs found:"+webelm.text)
            else:
                raise Exception("Userlinks did not match: ltext:"+ltext+" vs found:"+webelm.text)
        except Exception, e:
            self.log.critical("error Validating user link:" + str(e))
            
        
    
    def getUserLink(self,linkstring =""):
        '''
        get the link user@account link at top of page...
        '''
        self.log.debug(self.hd("getUserLink starting..."))
        driver = self.driver
        try:
            if (linkstring == "" ):
                #use xpath
                elm = driver.find_element_by_xpath(self.wg.xUserLink)
            else:
                #use the name@domain link string
                elm = driver.find_element_by_link_text(linkstring)
            self.log.debug("got elm for user link")
            return elm
        except Exception, e:
            self.log.critical("Failed to get User Link")
            raise e
        
        
    def downloadCredentials(self, username, accountname, timeout=60, force = False, callBackMethod=None):
        '''
        hack to temporarily simulate a user downloading creds
        note this will always attempt to download to the default dir setup when creating the webdriver instance
        will lock the directory until the method passed is executed on the file, note method will be passed the file name/path as the single arg
        hack work around to allow multiple instances of this test to run  at the same time
        '''
        try:
            driver = self.driver
            self.getUserLink().click()
            retcode = 0
            waitforlock = timeout
            mypid = str(os.getpid())
            mylockfile = self.downloadPath+'/lockfile'+str(mypid)
            #spin here wait for lockfile up to timeout
            while ((retcode == 0) and (waitforlock > 0)):
                retcode = call('ls '+self.downloadPath+' | grep -i lockfile',shell=True)
                if (retcode == 0):
                    self.log.debug("download dir is locked, waitforlock:"+str(waitforlock))
                time.sleep(1)
                waitforlock -= 1
            #check to see if we timed out or really go the go ahead
            if (retcode == 0):
                if (force):
                    self.log.critical("download dir was not empty after("+str(timeout)+")seconds, forcing removal...")
                    cmd = 'rm '+self.downloadPath+'/lockfile*'
                    self.log.critical("using cmd:"+cmd)
                    call(cmd,shell = True)
                    retcode = call('ls '+self.downloadPath+' | grep -i lockfile',shell=True)
                    if (retcode == 0):
                        raise Exception("download dir still not empty after force removal")
                else:
                    raise Exception("download dir:"+self.downloadPath+" was not empty after("+str(timeout)+") seconds")
            #create a lockfile in the download dir
            call("touch "+mylockfile, shell=True)
            #clear out the directory of previous zip files
            call("rm -f "+self.downloadPath+"/*.zip",shell=True)
            
            #now download the file via the webgui...
            driver.find_element_by_link_text("Download new credentials").click()
            #now check the local filesystem under the default dir to see if it was downloaded...
            downloaded = 1
            waitforfile = timeout
            #wait for the timeout period for the file to show up in our download dir...
            while ((downloaded == 1) and (waitforfile > 0)):
                downloaded = call('ls -la '+self.downloadPath+' | grep -i zip',shell=True)
                time.sleep(1)
                waitforfile -= 1
            if (downloaded == 1):
                raise Exception("File was not detected in("+self.downloadPath+", waitforfile:"+str(waitforfile))
            #get the file name since all we know so far is based on the assumption the file is named with a .zip extension... 
            filelist = os.listdir(self.downloadPath+"/")
            credfile = ""
            for f in filelist:
                if (re.search('zip$',f)):
                    credfile = self.downloadPath+"/"+f
                    self.log.debug("file found:"+str(credfile))
            if (credfile == ""):
                raise Exception("Failed to create the downloaded filename?")
            try:
                if (callBackMethod is not None):
                    callBackMethod(credfile)
            except Exception, cbe:
                raise Exception("Call back ("+str(callBackMethod.__name__)+") Failed: " +str(cbe))
        except Exception, e:
            self.log.debug("downloadCredentials failed:" + str(e))
        finally:
            try:
                self.log.debug("Removing file: "+credfile)
                os.remove(credfile)
            except Exception, rme:
                self.log.debug("Exception caught removing files:" + str(rme))
                pass
            try: 
                self.log.debug("Removing file: "+mylockfile)
                os.remove(mylockfile)
            except Exception, rme:
                self.log.debug("Exception caught removing files:" + str(rme))
                pass
            
            
            
    def unzipCredentialsToDir(self,sourcefile,destdir):
        '''
        unzip downloaded creds from 'sourcefile' to 'destdir'
        '''
        try:
            self.log.debug("unzipCredentialsToDir, sourcefile:"+str(sourcefile)+" destdir:"+str(destdir))
            #unzip credentials from a source path/file to a destdir
            cmd = 'unzip -o '+str(sourcefile)+' -d '+str(destdir)
            #status, output = commands.getstatusoutput(cmd)
            #self.log.debug("cmd:"+str(cmd)+", output:"+output)
            status = call(cmd, shell=True)
            if (status != 0 ):
                raise Exception("Failed to extract credentials for zip file")
        except Exception, e:
            raise Exception("unzipCredentialsToDir Failed: "+str(e))
        
    def sourceEucarcFromDir(self,srcdir):
        try:
            srcdir = str(srcdir)
            self.log.debug("sourceEucarcFromDir srcdir: " + srcdir)
            srcdir = str(srcdir)
            filelist = os.listdir(str(srcdir))
            if (not re.search('/$',srcdir) ):
                srcdir = srcdir+'/'
            for f in filelist:
                self.log.debug("file in dir:"+ str(f))
                if (re.search('eucarc',f)):
                    cmd = 'source '+srcdir+f
                    self.log.debug("exec cmd:"+str(cmd))
                    status, output = commands.getstatusoutput(cmd)
                    self.log.debug("cmd output:"+str(output))
                    if (status != 0 ):
                        raise Exception("cmd("+cmd+")failed with status:"+str(status))
                    return
            raise Exception("eucarc file not found in dir:"+str(srcdir))
        except Exception, e:
            raise Exception("sourceEucarcFromDir failed:" + str(e))
        
 
    def confirmAccount(self,acctname):
        '''
        This should simulate the user getting the email and clicking the confirmation link
        after being approved
        '''
        try:
            acctname = str(acctname)
            driver = self.driver
            link = self.getApprovalEmailLink(acctname)
            self.driverRestart()
            self.driver.get("about:blank")
            driver.get(str(link))
            self.waitForTextOnPage("confirmed", 2)
            driver.find_element_by_css_selector("button.gwt-Button").click()
        except Exception, e:
            self.log.critical("Could not confirm account:"+str(e))
            raise e
        #clear the browser since this sequence of events ends here
        self.driverRestart()
    
    def getApprovalEmailLink(self,acctname):
        #ssh into clc and see if an email and link was generated for this acctname
        acctname = str(acctname)
        output = []
        line = ""
        self.log.debug(self.hd("checkApprovalEmail starting("+acctname+")"))
        #ugly but simple way to get the last emailed link for this account from clc...
        searchcmd = "cat /var/spool/mail/root | grep \"'"+acctname+"' application was approved\" -a3 | grep https | tail -1"
        self.log.debug("checkApprovalEmail using cmd:"+str(searchcmd))
        try:
            output = self.eutester.sys(str(searchcmd))
        except Exception, e:
            self.log.critical("doh! error with ssh cmd: "+ str(e))
            raise e
        #this should probably only be 1 line but do this to make changing the search cmd easier if needed...?
        if (output != []):
            for line in output:
                self.log.debug("line in output:"+str(line))
                m = re.search('https', line)
                if (m):
                    self.log.debug("got Match:"+str(line))
                    return line
        raise Exception("No approval link found in email?")
        
    def getPasswordResetEmailLink(self,user,account):
        #ssh into clc and see if an email and link was generated for this acctname
        account = str(account)
        user = str(user)
        output = []
        line = ""
        self.log.debug(self.hd("getPasswordResetEmailLink starting("+user+"@"+account+")"))
        #ugly but simple way to get emailed link from clc...
        searchcmd = "cat /var/spool/mail/root | grep '"+user+"@"+account+"' -a10 | grep 'reset:confirmationcode' | tail -1"
        self.log.debug("getPasswordResetEmail using cmd:"+str(searchcmd))
        try:
            output = self.eutester.sys(str(searchcmd))
        except Exception, e:
            self.log.critical("doh! error with ssh cmd: "+ str(e))
            raise e
        #this should probably only be 1 line but do this to make changing the search cmd easier if needed...?
        if (output != []):
            for line in output:
                self.log.debug("line in output:"+str(line))
                m = re.search('https', line)
                if (m):
                    self.log.debug("got Match:"+str(line))
                    return line
        raise Exception("No password reset link found in email?")
    
    

    def changePassword(self,user,account,newpass,verify_pass="", expect=""):
        '''
        Simulate a user requesting a password reset, by clicking on reset link, getting email, 
        then clicking on emailed reset form, filling it out, and reseting password
        '''
        if (verify_pass == ""):
            verify_pass = newpass
        if (expect == ""):
            expect = self.wg.PasswordRestSuccess
        try:
            user = str(user)
            account = str(account)
            driver = self.driver
            link = self.getPasswordResetEmailLink(user, account)
            self.driverRestart()
            self.driver.get("about:blank")
            driver.get(str(link))
            self.waitForTextOnPage(self.wg.PasswordResetForm,2)
            driver.find_element_by_css_selector("input.gwt-PasswordTextBox").clear()
            driver.find_element_by_css_selector("input.gwt-PasswordTextBox").send_keys(newpass)
            driver.find_element_by_xpath("//tr[2]/td[2]/input").clear()
            driver.find_element_by_xpath("//tr[2]/td[2]/input").send_keys(verify_pass)
            driver.find_element_by_link_text("OK").click()
            self.waitForTextOnPage(expect, 0)
            if (expect == self.wg.PasswordRestSuccess):
                driver.find_element_by_css_selector("button.gwt-Button").click()
    
        except Exception, e:
            self.log.critical("Error with reset passwd dialog:"+str(e))
            raise e
        #clear the browser since this sequence of events ends here
        self.driverRestart()


    def signUpUser(self, account, user, passwd, email,expect= ""):
        '''
        Method is used to Sign a user up from the signup link on the login page
        Should fill out the form on the signup pop up and verify page returns to login page 
        afterwards.
        If these xpaths get used elsewhere should move to globals, otherwise
        this small portion may not be worth resolving xpath in globals since it's unique to this function
        '''
        if (expect == ""):
            expect = self.wg.AccountSignUpSuccess
        self.log.debug(self.hd("signUpUser:account:"+account+", user:"+ user+", passwd:"+ passwd+", email:"+email))
        try:
            driver = self.driver
            driver.get(self.clcurl) # assume were logged in, and go to home page...
            driver.find_element_by_link_text("Signup Account").click()
            driver.find_element_by_css_selector("input.gwt-TextBox").clear()
            driver.find_element_by_css_selector("input.gwt-TextBox").send_keys(account)
            driver.find_element_by_xpath("//tr[2]/td[2]/input").clear()
            driver.find_element_by_xpath("//tr[2]/td[2]/input").send_keys(email)
            driver.find_element_by_css_selector("input.gwt-PasswordTextBox").clear()
            driver.find_element_by_css_selector("input.gwt-PasswordTextBox").send_keys(passwd)
            driver.find_element_by_xpath("//tr[4]/td[2]/input").clear()
            driver.find_element_by_xpath("//tr[4]/td[2]/input").send_keys(passwd)
            driver.find_element_by_link_text("OK").click()
            self.waitForTextOnPage(expect,1) 
            if (expect == self.wg.AccountSignUpSuccess):
                driver.find_element_by_css_selector("button.gwt-Button").click()
                #wait for password on the page to verify the login page is returned
                #use a retry count of 3 which should essentially wait for 3 x the implicit wait period
                self.waitForText(self.wg.xPage,"Password", 3)#return to home page failed
        except Exception, e:
            self.log.critical("Err caught during SignUpUser ("+user+"@"+account+") from login page:\n"+ str(e))
            raise e
        

    def clickButtonXpathAndVerify(self,xpath,verify_text):
        '''
        click button at xpath and verify the success 
        by checking the page for an expected string verify_text
        selenium will error if link is not on the visible page, and scrolling/resizing is not working
        in the latest version for FF and IE...?
        '''
        try:
            self.log.debug(self.hd("clickButtonXpathAndVerify starting...\nButtonxpath:"+str(xpath)+"\nVerify_text:"+verify_text))
            driver = self.driver
            button=xpath
            driver.find_element_by_xpath(button).click()
            self.waitForTextOnPage(verify_text)
        except Exception, e:
            self.log.critical("clickButtonXpathAndVerify err:"+str(e))
            raise e
        
    def getAccountListAll(self):
        self.log.critical(self.hd("gettAccountListALL starting..."))
        return self.getAccountList(None,None,None)
    
    def getAccountById(self,uid):
        self.log.critical(self.hd("Starting getAccountById:"+str(uid)))
        if (not uid):
            self.log.critical("uid was null in getAccountById")
            raise Exception("uid was null in getAccountById")
        else:
            return self.getAccountList(None, uid, None)
            
            
    def getAccountByName(self, name):
        acct = euwebaccount.Euwebaccount()
        self.log.critical(self.hd("getAccountByName ("+str(name)+") starting...."))
        if (not name):
            self.log.critical("name was null in getAccountById")
            raise Exception("name was null in getAccountById")
        else:
            acct = self.getAccountList(name,None, None)
            if (acct is None):
                raise Exception("getAccountByName, accountlist returned null?")
            self.log.debug("Got account by name:")
            acct.printSelf()
            return acct
            
    def getAccountsByStat(self,stat):
        self.log.critical(self.hd("getAccountsByStat Starting:"+str(stat)))
        if (not stat):
            self.log.critical("Name was Null in getAccountsByStat")
            raise Exception("Name was null in getAccountsByStat")
        else:
            return self.getAccountList(None, None, stat)
        
    
    def getAccountList(self,name,uid,status):
        '''
        (avoid this method)
        ugly function in the making to traverse the accounts table and handle all the timing and stale page issues you can run into...
        navigate to the accounts page and get all the rows found in the accounts table   
        The list can be sorted by name, id or status, or if all are set to non it will retrieve them all
        '''
        
        self.log.debug(self.hd("getAccountList starting, name:("+str(name)+") id:("+str(uid)+") status:("+str(status)+")"))
        if (name is not None):
            name = str(name)
        if (uid is not None):
            uid = str(uid)
        if (status is not None):
            status = str(status)
            
        if ((name is None) and (uid is None) and (status is None)):
            getall = True
        else:
            getall = False
        try:
            accountlist = []
            driver = self.driver
            #navigate to page
            self.clickButtonXpathAndVerify( self.wg.xLn_Accounts, self.wg.AccountsTxt)
            self.driver.refresh()# 
            #make sure the table is present
            try:
                driver.find_elements_by_xpath(self.wg.xAccountsTable)
            except Exception, e:
                self.debug("accounts list is empty:"+ str(e))
                return accountlist
            prange = str(driver.find_element_by_xpath(self.wg.xAccountPageRange).get_attribute("textContent"))
            accountsonpage = int(prange.split('of').pop())
            row_xpath=self.wg.xAccountsRow1
            tr = 1
            page = 1
            retry = 1
            driver.implicitly_wait(1)
            #loop over a known pattern in xpath to grab the contents of the table
            #From this we can grab account name, id, and status, also can hold 
            while (True):
                try:
                    if (tr == 1 ):
                        self.log.debug("starting search account loop on table(tr:"+str(tr)+")")
                    gAccount = euwebaccount.Euwebaccount() 
                    #account= driver.find_element_by_xpath(row_xpath)
                    gAccount.row_xpath = row_xpath
                    tr+=1
                    if (tr > 2):
                        retry = 1 #reset retry if we get this far
                    row_xpath = self.wg.xAccountsRow1+"["+str(tr)+"]"
                    id_xpath = row_xpath+'/td'
                    name_xpath = row_xpath+'/td[2]'
                    status_xpath = row_xpath+'/td[3]'
                    
                    gAccount.id  = driver.find_element_by_xpath(id_xpath).get_attribute("textContent")
                    gAccount.id_xpath = id_xpath
                    self.log.debug("got id:"+gAccount.id)
                    if ((not getall) and (uid != None)):
                        if( uid == gAccount.id):
                            addtolist = True
                        else:
                            addtolist = False
                            continue
                        
                    gAccount.name = driver.find_element_by_xpath(name_xpath).get_attribute("textContent")
                    gAccount.name_xpath = name_xpath
                    self.log.debug("got name:"+gAccount.name)
                    if ((not getall) and (name != None)):
                        if (name == gAccount.name):
                            addtolist = True
                        else:
                            addtolist = False
                            continue
                    
                    gAccount.status = driver.find_element_by_xpath(status_xpath).get_attribute("textContent")
                    gAccount.status_elm = status_xpath
                    self.log.debug("got status:"+gAccount.status)
                    if ((not getall) and (status != None)):
                        if (status == gAccount.status):
                            addtolist = True
                        else:
                            addtolist = False
                            continue
                    
                    #if we only expect to match 1 name or 1 id return the account obj if we have one
                    if (addtolist and (not getall) and (not status)):
                        self.log.debug("returning single account (Acct#:"+str(tr)+" on page:"+str(page)+") :"+gAccount.name)
                        return gAccount
                        
                    #Otherwise if were getting em all or sorting to create a list add append it to the list
                    if (getall or addtolist):
                        self.log.debug("Adding account(#"+str(tr)+") to list")
                        accountlist.append(gAccount)
                    
                       
                except NoSuchElementException:
                    print "No such elm in list. tr:"+str(tr)+" retry:"+str(retry)+" accountsonpage:"+str(accountsonpage)
                    if ((accountsonpage > 0) and (retry < 5) and (tr == 2)):
                            self.log.debug("Total Accounts:"+str(accountsonpage)+" != 0, but found 0 here. Retry:"+str(retry)+" tr:"+str(tr))
                            retry += 1
                            tr = 1
                            time.sleep(3)
                    else:
                        try:
                            #We may have more accounts on the next page so try the forward buttton first...
                            self.log.debug("end of list try pressing the forward button (tr:"+str(tr)+" retry:"+str(retry)+" ...")
                            prange = str(driver.find_element_by_xpath(self.wg.xAccountPageRange).get_attribute("textContent"))
                            accountsonpage = int(prange.split('of').pop())
                            #click our button, and expect to NOT find range, if we do the button failed and were on the same page
                            try:
                                driver.implicitly_wait(3)
                                self.clickButtonXpathAndVerify(self.wg.xAccountForwardButton, prange)
                                button_failed = True
                            except Exception, be:
                                self.log.debug("range("+prange+")not found, assuming page progressed=good"+str(be))
                                button_failed = False
                                pass
                            if (button_failed):
                                raise Exception("Forward button did not progress page")
                            
                            row_xpath=self.wg.xAccountsRow1
                            tr = 1
                            page += 1
                            #Make sure the table gets reloaded:
                            driver.find_elements_by_xpath(self.wg.xAccountsTable)
                            driver.find_element_by_xpath(row_xpath)
                            time.sleep(1)
                            
                            self.log.debug("Pressed button, looks like there's more?")
                            driver.implicitly_wait(1)
                        except Exception, e:
                            self.log.debug("NoSuchElement assuming end of list found")
                            if (accountlist == []):
                                raise Exception("account not found(name:"+str(name)+" id:"+str(uid)+" status:"+str(status)+" )list is empty")
                            else:
                                self.log.debug("Returning account list?")
                                return accountlist
                
            raise Exception("uh oh, out of the loop?")
        except Exception, e:
            self.log.critical("Err while getting account list:"+str(e))
            raise e
        finally:
            driver.implicitly_wait(10)
            
            
  
    def searchForElementByUrl(self,url):
        '''
        Enter search criteria into url, this will be appended to the base clc url. 
        see euwebglobals for help with examples
        '''
        url = self.clcurl+str(url)
        self.log.debug(self.hd("searchForElementByUrl ("+str(url)+")"))
        #strings = verifytext = url.split('=')
        driver = self.driver
        driver.get(url)
        self.log.debug("search results page loaded, now verify search criteria is on page...")
        #now got through the url and verify all the seach criteria is present on the screen...
        pairs = re.findall('\w+\=\w+',url)
        for p in pairs:
            verifytext = p.split('=').pop()
            self.log.debug("searchForElementByUrl verifytext:"+verifytext)
            self.waitForTextOnPage(verifytext, 2)
        self.log.debug("searchForElement:Found, should be present in screen...")
        
    
    def searchForUser(self,user,account):
        '''
        Formats a user specific query to feed to search by url
        '''
        user=str(user)
        account=str(account)
        url = self.wg.urlSearchUserByAccount+account+" name="+user
        self.log.debug("searchForUser Url ("+str(url)+")")
        self.searchForElementByUrl(str(url))
    

    def getUserRowValuesByXpath(self,row_xpath):
        '''
        Assumes the table is current within the browser
        note this only gets the row values, so does not completely populate the user object
        '''
        try:
            self.log.debug(self.hd("getUserValuesByXpath: " +str(row_xpath)))
            driver = self.driver
            user = euwebuser.Euwebuser(makerandom=False)
            #first make sure the row is available...
            driver.find_element_by_xpath(row_xpath)
            
            id_xpath = row_xpath+'/td'
            name_xpath = row_xpath+'/td[2]'
            path_xpath = row_xpath+'/td[3]'
            account_xpath = row_xpath+'/td[4]'
            enabled_xpath = row_xpath+'/td[5]'
            status_xpath = row_xpath+'/td[6]'
            
            user.id = driver.find_element_by_xpath(id_xpath).get_attribute("textContent")
            user.user = driver.find_element_by_xpath(name_xpath).get_attribute("textContent")
            user.path = driver.find_element_by_xpath(path_xpath).get_attribute("textContent")
            user.account = driver.find_element_by_xpath(account_xpath).get_attribute("textContent")
            user.enabled = driver.find_element_by_xpath(enabled_xpath).get_attribute("textContent")
            user.status = driver.find_element_by_xpath(status_xpath).get_attribute("textContent")
            user.printUser()
            return user
        except Exception, e:
            raise Exception("Failed to get user values from table: "+ str(e))
            
            
            
        
        

    def enableDisableUser(self,user,account,enable_flag=True):
        '''
        Bring user into focus and toggle the rightnav element based on the enable value
        verifies the change has taken affect. 
        '''
        driver = self.driver
        if (enable_flag):
            entxt = "true"
        else:
            entxt = "false"
        self.log.debug(self.hd("enableDisableUser: user:"+user+" enable_flag: "+str(enable_flag)) )
        #search for user, user should be only element in the table after this...
        self.searchForUser(user,account)
        #bring user into focus, which brings up right nav
        self.clickButtonXpathAndVerify(self.wg.xUsersRow1Id, user)
        state = self.getRightNavValueByName(self.wg.UsersRightNav_Enabled, attribute="checked")
        self.log.debug("current state is: "+state+" , desired state:"+entxt)
        if ((state == "true")==enable_flag):
            self.log.debug("Enabled state is already set to:"+str(state))
            return
        self.log.debug("Attempting to click checkbox into desired state: "+entxt)
        #toggle state of checkbox...
        checkBoxXpath = self.getRightNavValueByName("Enabled",returnXpath = True)
   
        driver.find_element_by_xpath(checkBoxXpath).click()
        time.sleep(2)
   
        self.log.debug("clicked check box, now click save...")
        time.sleep(5)
        #save it
        driver.find_element_by_xpath(self.wg.xRighNavSaveButton).click()
        #verify the change took affect...
        self.log.debug("clicked save, now refresh and check that it took effect")
        time.sleep(5)
        driver.refresh()
        #bring user back into focus, assuming still on the user page...
        self.clickButtonXpathAndVerify(self.wg.xUsersRow1Id, user)
        newstate = self.getRightNavValueByName(self.wg.UsersRightNav_Enabled, attribute="checked") 
        if (newstate == state):
            raise Exception("Enabled state checkbox value did not change was:"+str(state)+" now: "+str(newstate))
        
        
    
    def addUserToAccount(self,accountname, username, path="", verifyLog = True):
        #assumes the current user is admin or has priv to do so...
        if (path == ""):
            path = "/"+username
        self.log.debug(self.hd("addUserToAccount (account:"+accountname+" user: "+username+" path:"+path+")"))
        driver = self.driver
        self.searchForElementByUrl(self.wg.urlSearchAccountByName+accountname)
        self.clickButtonXpathAndVerify(self.wg.xAccountsRow1Id, self.wg.rightNavTxt)
        driver.find_element_by_link_text("New users").click()
        driver.find_element_by_xpath("//td/div/table/tbody/tr/td[2]/input").clear()
        driver.find_element_by_xpath("//td/div/table/tbody/tr/td[2]/input").send_keys(username)
        driver.find_element_by_xpath("//td/div/table/tbody/tr[2]/td[2]/input").clear()
        driver.find_element_by_xpath("//td/div/table/tbody/tr[2]/td[2]/input").send_keys(path)
        driver.find_element_by_link_text("OK").click()
        if (verifyLog):
            result = self.getOnScreenLogMesg()
            if (result == "Failed to create users" ):
                raise Exception("Log indicates: Failed to create account")
            else:
                self.log.debug("log message after addUserToAccount:"+result)
                
                
    
    def changeUserPassword(self, accountname, username, adminpass, newpass, expect ="", vpass="", verifyLog=True):
        #assumes that this is executed from the admin user of the specific account this user is part of
        #changes the password of a user, will wait for text "expect" if used
        if (vpass == ""):
            vpass =newpass # used in the case you wan the 2nd typed passwd to be different from the 1st for testing
        self.log.debug(self.hd("changeUserPassword: (account:"+accountname+" user: "+username+" oldpass:"+adminpass+" newpass:"+newpass+")"))
        driver= self.driver
        url = self.wg.urlSearchUserByAccount+accountname+" name="+username
        self.searchForElementByUrl(url)
        self.clickButtonXpathAndVerify(self.wg.xUsersRow1Id, self.wg.rightNavTxt)
        passwdxpath = self.getRightNavValueByName(self.wg.UsersRighNav_Password,attribute="tagName",returnXpath=True, input_path="/div/a/span")
        driver.find_element_by_xpath(passwdxpath).click()
        self.waitForTextOnPage("Change password", 1)
        driver.find_element_by_css_selector("input.gwt-PasswordTextBox").clear()
        driver.find_element_by_css_selector("input.gwt-PasswordTextBox").send_keys(adminpass)
        driver.find_element_by_xpath("//td/div/table/tbody/tr[2]/td[2]/input").clear()
        driver.find_element_by_xpath("//td/div/table/tbody/tr[2]/td[2]/input").send_keys(newpass)
        driver.find_element_by_xpath("//tr[3]/td[2]/input").clear()
        driver.find_element_by_xpath("//tr[3]/td[2]/input").send_keys(vpass)
        driver.find_element_by_link_text("OK").click()
        if (expect != ""):
            self.waitForTextOnPage(expect, 1)
        if (verifyLog == True):
            result = self.getOnScreenLogMesg()
            self.log.debug("change passwd log result text:"+result)
            if (result == "Failed to change password"):
                raise Exception("Log Msg indicates we failed to Change passoword")
            elif ( result == "Password changed"):
                self.log.debug("Password Changed successfully")
            else:
                self.log.critical("Unrecognized or missing log message during changeUserPassword")
                
                
    
    def getOnScreenLogMesg(self):
        #attempt to get the most recent log message to the screen
        try:
            message = self.driver.find_element_by_xpath(self.wg.xLogMessage).get_attribute("textContent")
            message = str(message)
            self.log.debug("GetOnScreenLogMesg returning: "+message)
            return message
        except NoSuchElementException:
            self.log.debug("NoSuchElement assuming there is no message")
            return ""
        
        
        
    def userFirstTimeLogin(self,email,oldpass,newpass,newpass2="",expect=""):
        '''
        Assumes the firttime login pop up is present, attempts to fill out the forms
        email - address to enter into form for this user
        oldpass - the expected current password for this user
        newpass - the newpassword for this user
        newpass2 - the text to be entered in the 'verify password' box, default is = newpass
        expect - any text to be expected on the page at the end of the execution 
        '''
        if (newpass2 == ""):
            newpass2 = newpass
        driver = self.driver
        try:
            driver.find_element_by_css_selector("input.gwt-TextBox").clear()
        except Exception, e:
            raise("firstTimelogin failed, firsttime popup may be missing?:"+str(e))
        driver.find_element_by_css_selector("input.gwt-TextBox").send_keys(email)
        driver.find_element_by_css_selector("input.gwt-PasswordTextBox").clear()
        driver.find_element_by_css_selector("input.gwt-PasswordTextBox").send_keys(oldpass)
        driver.find_element_by_xpath("//tr[3]/td[2]/input").clear()
        driver.find_element_by_xpath("//tr[3]/td[2]/input").send_keys(newpass)
        driver.find_element_by_xpath("//tr[4]/td[2]/input").clear()
        driver.find_element_by_xpath("//tr[4]/td[2]/input").send_keys(newpass2)
        driver.find_element_by_link_text("OK").click()
        if (expect != ""):
            self.waitForTextOnPage(expect, 1)
        
        
    def deleteAccount(self,accountname,expect="",verifylog=True):
        driver = self.driver
        self.searchForElementByUrl(self.wg.urlSearchAccountByName+accountname)
        driver.find_element_by_link_text("Delete accounts").click()
        driver.find_element_by_link_text("OK").click()
        if (verifylog == True):
            result = self.getOnScreenLogMesg()
            if (result != "Accounts deleted" ):
                raise Exception("Log indicates: Failed to create account")
            else:
                self.log.debug("log message after addUserToAccount:"+result)
        

 
    def clickAndApproveAccount(self, bool_approve, bool_ok):
        #this assumes the test has already clicked on the xpath of a user item to bring that
        #user into focus on the accounts page. ok boolean indicates how the popup window is handled
        driver = self.driver
        if (bool_approve):
            #click on the approve button
            driver.find_element_by_link_text("Approve").click()
        else:
            driver.find_element_by_link_text("Reject").click()
        #now handle the popup which follows...
        if (bool_ok):
            #click on the ok button in the popup
            driver.find_element_by_link_text("OK").click()
        else:
            #click on the cancel button on the popup
            driver.find_element_by_link_text("Reject").click()
        driver.refresh()
    
    
    def addNewGroupToAccount(self, accountname,newgroupname, path="", verifyLog=True):
        #selects an account and proceeds to add a new group and group path to it
        if (path == ""):
            path = "/"+newgroupname
        driver = self.driver
        #bring our account into focus and select it
        self.searchForElementByUrl(self.wg.urlSearchAccountByName+accountname)
        self.driver.find_element_by_xpath(self.wg.xAccountsRow1Id).click()
        driver.find_element_by_link_text("New groups").click()
        driver.find_element_by_xpath("//td/div/table/tbody/tr/td[2]/input").clear()
        driver.find_element_by_xpath("//td/div/table/tbody/tr/td[2]/input").send_keys(newgroupname)
        driver.find_element_by_xpath("//td/div/table/tbody/tr[2]/td[2]/input").clear()
        driver.find_element_by_xpath("//td/div/table/tbody/tr[2]/td[2]/input").send_keys(path)
        driver.find_element_by_xpath("//tr[4]/td/a/span").click()
        #check the log at bottom of screen
        time.sleep(1) #give a second before fetching log message
        if (verifyLog == True):
            result = self.getOnScreenLogMesg()
            if (result.lower().find('fail') >= 0) :
                self.log.critical("Log Message:("+result+")")
                raise Exception("Log indicates: Failed to add group "+newgroupname+" to account")
            else:
                self.log.debug("log message after addNewGroupToAccount:("+result+")")
        
        
    def addExistingUserToGroup(self, accountname, groupname,username, verifyLog=True):
        self.log.debug( self.hd("addExistingUserToGroup starting (accountname:"+str(accountname)+" groupname:"+str(groupname)+" username:"+str(username)+")") )
        driver=self.driver
        #get group into focus and selected
        url = self.wg.urlSearchGroupByName+groupname+" account="+accountname
        self.searchForElementByUrl(url)
        time.sleep(3) #let page load
        self.log.debug("attempting to click on group row")
        driver.find_element_by_xpath(self.wg.xGroupRow1Id).click()
        self.log.debug("clicked on group row ,now looking for add users link to click")
        
        driver.find_element_by_link_text("Add users").click()
        self.log.debug("just clicked add users link")
        driver.find_element_by_xpath("//td/div/table/tbody/tr/td[2]/input").clear()
        driver.find_element_by_xpath("//td/div/table/tbody/tr/td[2]/input").send_keys(username)
        #driver.find_element_by_xpath("/html/body/div[6]/div/table/tbody/tr[2]/td[2]/div/div/table/tbody/tr[4]/td/a/span").click()
        driver.find_element_by_xpath("//html/body/div[6]/div/table/tbody/tr[2]/td[2]/div/div/table/tbody/tr[4]/td/a").click()
        #check the log at bottom of screen
        time.sleep(1) # give a second for the log message to update
        if (verifyLog == True):
            result = self.getOnScreenLogMesg()
            if (result.lower().find('fail') >= 0):
                self.log.critical("Log Message:("+result+")")
                raise Exception("Log indicates: Failed to add group "+groupname+" to account")
            else:
                self.log.debug("log message after addNewGroupToAccount:("+result+")")
        
    def addPolicyToAccount(self, accountname, newpolicyname, policytext, verifyLog=True):
        self.log.debug(self.hd("addPolicyToAccount(accountname:"+str(accountname)+" policytext:"+str(policytext)+")"))
        driver = self.driver
        #bring our account into focus and select it
        self.searchForElementByUrl(self.wg.urlSearchAccountByName+accountname)
        self.driver.find_element_by_xpath(self.wg.xAccountsRow1Id).click()
        #bring up add policy pop up window
        driver.find_element_by_link_text("Add policy").click()
        #fill out popup window add policy form
        driver.find_element_by_xpath("//td/div/table/tbody/tr/td[2]/input").clear()
        driver.find_element_by_xpath("//td/div/table/tbody/tr/td[2]/input").send_keys(newpolicyname)
        driver.find_element_by_css_selector("textarea.gwt-TextArea").clear()
        #enter our policy text string...
        #driver.find_element_by_css_selector("textarea.gwt-TextArea").send_keys("{\n  \"Statement\": [\n    {\n      \"Sid\": \"Stmt1323987968698\",\n      \"Action\": \"*\",\n      \"Effect\": \"Deny\",\n      \"Resource\": \"*\"\n    }\n  ]\n}")
        driver.find_element_by_css_selector("textarea.gwt-TextArea").send_keys(policytext)
        driver.find_element_by_link_text("OK").click()
        time.sleep(1) # give a second for the log message to update
        if (verifyLog == True):
            result = self.getOnScreenLogMesg()
            if (result.lower().find('fail') >= 0):
                self.log.critical("Log Message:("+result+")")
                raise Exception("Log indicates: Failed to add policy:("+str(newpolicyname)+") to account:("+str(accountname)+")")
            else:
                self.log.debug("log message after addPolicyToAccount:("+result+")")
    
    def addPolicyToUser(self,username,accountname,newpolicyname, policytext, verifyLog=True):
        self.log.debug(self.hd("addPolicyToUser(accountname:"+str(accountname)+" policytext:"+str(policytext)+")"))
        driver = self.driver
        #bring our user into focus and select it
        url = self.wg.urlSearchUserByName+username+" account:"+accountname
        self.searchForElementByUrl(url)
        self.driver.find_element_by_xpath(self.wg.xUsersRow1Id).click()
        #bring up add policy pop up window
        driver.find_element_by_link_text("Add policy").click()
        #fill out popup window add policy form
        driver.find_element_by_xpath("//td/div/table/tbody/tr/td[2]/input").clear()
        driver.find_element_by_xpath("//td/div/table/tbody/tr/td[2]/input").send_keys(newpolicyname)
        driver.find_element_by_css_selector("textarea.gwt-TextArea").clear()
        #enter our policy text string...
        #driver.find_element_by_css_selector("textarea.gwt-TextArea").send_keys("{\n  \"Statement\": [\n    {\n      \"Sid\": \"Stmt1323987968698\",\n      \"Action\": \"*\",\n      \"Effect\": \"Deny\",\n      \"Resource\": \"*\"\n    }\n  ]\n}")
        driver.find_element_by_css_selector("textarea.gwt-TextArea").send_keys(policytext)
        driver.find_element_by_link_text("OK").click()
        time.sleep(1) # give a second for the log message to update
        if (verifyLog == True):
            result = self.getOnScreenLogMesg()
            if (result.lower().find('fail') >= 0):
                self.log.critical("Log Message:("+result+")")
                raise Exception("Log indicates: Failed to add policy:("+str(newpolicyname)+") to "+str(username)+"@"+str(accountname)+")")
            else:
                self.log.debug("log message after addPolicyToUser:("+result+")")
    
    def addPolicyToGroup(self,groupname,accountname, newpolicyname, policytext, verifyLog=True):
        self.log.debug(self.hd("addPolicyToGroup(groupname:"+str(accountname)+" policytext:"+str(policytext)+")"))
        driver = self.driver
        #bring our group into focus and select it
        url = self.wg.urlSearchGroupByName+groupname+" account:"+accountname
        self.searchForElementByUrl(url)
        self.driver.find_element_by_xpath(self.wg.xGroupRow1Id).click()
        #bring up add policy pop up window
        driver.find_element_by_link_text("Add policy").click()
        #fill out popup window add policy form
        driver.find_element_by_xpath("//td/div/table/tbody/tr/td[2]/input").clear()
        driver.find_element_by_xpath("//td/div/table/tbody/tr/td[2]/input").send_keys(newpolicyname)
        driver.find_element_by_css_selector("textarea.gwt-TextArea").clear()
        #enter our policy text string...
        #driver.find_element_by_css_selector("textarea.gwt-TextArea").send_keys("{\n  \"Statement\": [\n    {\n      \"Sid\": \"Stmt1323987968698\",\n      \"Action\": \"*\",\n      \"Effect\": \"Deny\",\n      \"Resource\": \"*\"\n    }\n  ]\n}")
        driver.find_element_by_css_selector("textarea.gwt-TextArea").send_keys(policytext)
        driver.find_element_by_link_text("OK").click()
        time.sleep(1) # give a second for the log message to update
        if (verifyLog == True):
            result = self.getOnScreenLogMesg()
            if (result.lower().find('fail') >= 0):
                self.log.critical("Log Message:("+result+")")
                raise Exception("Log indicates: Failed to add policy:("+str(newpolicyname)+") to "+str(groupname)+"@"+str(accountname)+")")
            else:
                self.log.debug("log message after addPolicyToGroup:("+result+")")
                
                
    def modifyVmType(self,vmtype, attr, value, verifyLog = True):
        '''
        modify an attribute of a vmtype named 'vmtype'
        attempt to set the vmtype's attribute 'attr' to 'value'
        Note: use euweb globals vm right nav txt for the attrs when using this method
        '''
        self.log.debug(self.hd("addPolicyToGroup(vmtype:("+str(vmtype)+") attribute:"+str(attr)+" value:"+str(value)+")"))
        driver = self.driver
        #got to the vm page
        driver.get(self.clcurl+self.wg.urlSearchVmType)
        self.waitForTextOnPage(self.wg.VmTypesTxt)
        #find our row based on the name of the vm type, and bring it into focus to open the right nav
        #modify the xpath so the first element in the row is what gets clicked
        driver.find_element_by_xpath(self.getTableRowXpathByString(self.wg.xVmTypeTable, vmtype)+"/td").click()
        #now look for a table elment with the exact text 'attr' and set it's value to value
        attrxpath = self.getRightNavValueByName(attr, returnXpath=True)
        driver.find_element_by_xpath(attrxpath).clear()
        driver.find_element_by_xpath(attrxpath).send_keys(str(value))     
        #then click save
        time.sleep(1)
        driver.find_element_by_xpath(self.wg.xRighNavSaveButton).click()
        time.sleep(1)
        if (verifyLog == True):
            result = self.getOnScreenLogMesg()
            if (result.lower().find('fail') >= 0):
                self.log.critical("Log Message:("+result+")")
                raise Exception("Log indicates: Failed vmtype modification:("+str(vmtype)+")  attr"+str(attr)+" to "+str(value)+")")
            else:
                self.log.debug("log message after vmtype modification:("+result+")")
        
        
    def getTableRowXpathByString(self, table_xpath, match, startcol=1, endcol=1, case=False, highlightrow=False):
        '''
        Traverse a table at tablexpath, if elment at 'startcol' through 'endcol' matches the string 'match' 
        then the xpath for the row which that was found in is returned 
        This method assumes the page containing the table at tablexpath is loaded.
        This method can be used with other methods which forward elements in the the table ie:forward/reverse display buttons
        -'case' determines case sensative matching. 
        -'highlight' row can be used for debug to see which row is being examined in the browser if the row supports click, the first element will be click()'d
        '''
        if (case == False):
            match = str(match).upper()
        table_xpath = str(table_xpath)
        self.log.debug(self.hd("getTableRowXpathByName( table xpath:("+table_xpath+") match:("+match+")"))
        driver = self.driver
        row_xpath = table_xpath+"/tbody/tr"
        tr = 1
        td = startcol
        try:
            driver.find_elements_by_xpath(table_xpath)
        except Exception, nfe:
            self.log.critical("Table not found at xpath:"+str(table_xpath))
            raise nfe
        #to speed up the interations through the test set our timeout to 1 second per element not found
        driver.implicitly_wait(1)
        try:
            while (True):
                td = startcol
                if (td > 1):
                    element_xpath = row_xpath+"/td["+str(td)+"]"
                else:
                    element_xpath = row_xpath+"/td"
                
                if (highlightrow == True):
                    #click on the first element in the row so it is brought into focus
                    self.driver.find_element_by_xpath(row_xpath+"/td").click()
                self.log.debug ("trying td:"+str(td)+"row:"+str(row_xpath))
                try:
                    while  ( td <= endcol ):
                        self.log.debug("looking for ("+match+") at xpath:"+str(element_xpath))
                        vstr = str(driver.find_element_by_xpath(element_xpath).get_attribute("textContent"))
                        if (case == False):
                            vstr = vstr.upper()
                        self.log.debug( "Got value at row:"+str(tr)+" col:"+str(td)+" (value= "+str(vstr)+")")
                        if (vstr == match):
                            self.log.debug("Got match:("+match+" == "+vstr+") returning this row's xpath")
                            return row_xpath
                        td+=1
                        element_xpath=row_xpath+"/td["+str(td)+"]"
                    self.log.debug("not found in this row")
                    tr+=1
                    row_xpath=row_xpath+"["+str(tr)+"]"
                except NoSuchElementException, nse:
                    #the columns given likely exceeded the columns in the row so no such element
                    #not found on that row within  the columns defined, increment row and try again
                    self.log.debug("not found in this row")
                    tr+=1
                    row_xpath=row_xpath+"["+str(tr)+"]"
        except NoSuchElementException, nse:
            self.log.debug("End of visible table, item not found: "+ match)
            driver.implicitly_wait(10)
            raise nse
        finally:
            driver.implicitly_wait(10)
        
            
        
    
    def getRightNavValueByName(self,name,attribute="value", table_xpath="",row1_xpath="",returnXpath=False, input_path="[2]/input"):
        '''
        Get a table value out of specific page's right nav (or a table) by matching the name of the table element
        This nav is only shown for the account which is highlighted in the main accounts page. 
        for example "Registration status" would likely have a value of CONFIRMED or REGISTERED
        if returnXpath flag is set, this will return the xpath of the value element...
        xpath example: return the  xpath of checkbox to be checked or the textbox to be filled
        Note:Change input path if this is not a input text field, etc..
        '''
        if (table_xpath == ""):
            table_xpath = self.wg.xRightNavTable
        if (row1_xpath == ""):
            row1_xpath = table_xpath+"/tbody/tr"
            
        try:
            name = str(name)
            nav_val=""
            self.log.debug(self.hd("getRightNavValueByName (name:"+str(name)+" returnxpath:"+str(returnXpath)+")..."))
            driver = self.driver
            #self.clickButtonXpathAndVerify( self.wg.xLn_Accounts, self.wg.AccountsTxt)
            #make sure the table is present
            try:
                driver.find_elements_by_xpath(table_xpath)
            except Exception, e:
                self.debug("accounts RighNav is empty:"+ str(e))
                return nav_val  
            #now go through the table, if we find an element who's name matches our string
            #return the value of that element
            row_xpath=row1_xpath+"/td"
            tr = 1
            #were depending on a timeout to find an xpath to determine a failure or when the end 
            #of the table is reached, at this point we've already waited above for the table to load
            #so set the timeout to  small value now...
            driver.implicitly_wait(1)
            try:
                while (True):
                    self.log.debug("looking for ("+name+") at xpath:"+row_xpath)
                    vstr = str(driver.find_element_by_xpath(row_xpath).get_attribute("textContent"))
                    self.log.debug( "Got right nav element text: = "+vstr)
                    if (vstr == name):
                        self.log.debug("got match on: "+vstr)
                        nav_xpath = row_xpath+input_path
                        self.log.debug( "looking for value at:"+nav_xpath)
                        #either return the xpath of the elm or the string value of the elm here...
                        try:
                            nav_val = str(driver.find_element_by_xpath(nav_xpath).get_attribute(attribute))
                        except NoSuchElementException, nse:
                            self.log.debug("No such element,trying xpath with span for checkbox maybe?")
                            nav_xpath = row_xpath+"[2]/span/input"
                            nav_val = str(driver.find_element_by_xpath(nav_xpath).get_attribute(attribute))
                        self.log.debug("Found element("+name+") value("+nav_val+")")
                        if (returnXpath):
                            self.log.debug("Returning right nav XPATH: "+nav_xpath)
                            return nav_xpath
                        else:
                            return nav_val
                    tr+=1
                    row_xpath=row1_xpath+"["+str(tr)+"]/td"
            except NoSuchElementException, nse:
                self.log.debug("End of righ nav table, item not found: "+name)
                driver.implicitly_wait(10)
                raise nse
            finally:
                driver.implicitly_wait(10)
        except Exception, e:
            self.log.critical("Error getting element from accounts right nav:"+name)
            raise e
        
    #Click on reset password link on login page, and follow flow of popup
    #Expect a certain string to test for different flows, by default expects a succesful form flow
    #this should generate an emailed link 
    def requestPasswordReset(self,user,account,email,expect=""):
        self.log.debug(self.hd("requestPasswordReset..."))
        if (expect == ""):
            expect = self.wg.PasswordResetTxt
        driver = self.driver
        driver.find_element_by_link_text("Reset Password").click()
        driver.find_element_by_css_selector("input.gwt-TextBox").clear()
        driver.find_element_by_css_selector("input.gwt-TextBox").send_keys(user)
        driver.find_element_by_xpath("//tr[2]/td[2]/input").clear()
        driver.find_element_by_xpath("//tr[2]/td[2]/input").send_keys(account)
        driver.find_element_by_xpath("//tr[3]/td[2]/input").clear()
        driver.find_element_by_xpath("//tr[3]/td[2]/input").send_keys(email)
        driver.find_element_by_link_text("OK").click()
        self.waitForTextOnPage(expect,0)
        if (expect == self.wg.PasswordResetTxt):
            driver.find_element_by_css_selector("button.gwt-Button").click()
            self.waitForTextOnPage(self.wg.PasswordTxt, 0)
        
    def driverRestart(self):
        self.log.debug(self.hd("driverRestart..."))
        self.driver.delete_all_cookies()
        self.driver.get("about:blank")
        #self.driver.refresh()
        #self.driver.stop_client()
        
        
    #tear down this webdriver    
    def tearDown(self, stayOpenAfterError):
        if ( stayOpenAfterError != 0 ):
            time.sleep(stayOpenAfterError)
        self.driver.quit()
        self.driver.stop_client()
        #self.logger.outhdlr.close()
            
    def sig_handler(self,sig):
        self.log.critical( 'caught signal:' + sig )
        self.tearDown(0)
         
    
    signal.signal(signal.SIGINT, sig_handler)
            

            
class LoginFailure(Exception):
    def __init__(self, value):
        self.value = value
    def __str__ (self):
        return repr(self.value)
            

                       

        
        
        
            
            
            
            
            
                


#!/usr/bin/env python
#
#
# Description:  This script encompasses test cases/modules concerning instance specific behavior and
#               features for Eucalyptus.  The test cases/modules that are executed can be
#               found in the script under the "tests" list.


import time
from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase

class MyTestCase(EutesterTestCase):
    def __init__(self, config_file=None, password=None):
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("--command", default="free")
        self.parser.add_argument("--component", default=None)
        self.get_args()
        # Setup basic eutester object
        self.tester = Eucaops( config_file=self.args.config_file, password=self.args.password,download_creds=False)

    def clean_method(self):
        pass

    def MyTestUnit(self):
        """
        A test description must go here......
        This test will simply run an instance and check that it is reachable via ssh
        """
        target_machines = []
        if not self.args.component:
            target_machines = self.tester.config['machines']
        else:
            target_machines = self.tester.get_component_machines(self.args.component)

        for machine in target_machines:
            if machine.distro.name is "vmware":
                continue
            machine.sys(self.args.command)

if __name__ == "__main__":
    testcase = MyTestCase()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list  "VolumeTagging", "InstanceTagging", "SnapshotTagging", "ImageTagging"
    list = testcase.args.tests or ["MyTestUnit"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)



#!/usr/bin/python

import time
from eucaops import Eucaops
from eucaops import EC2ops
from eutester.eutestcase import EutesterTestCase
import os
import random

class InstanceBasics(EutesterTestCase):
    def __init__(self, extra_args= None):
        self.setuptestcase()
        self.setup_parser()
        if extra_args:
            for arg in extra_args:
                self.parser.add_argument(arg)
        self.get_args()
        # Setup basic eutester object
        if self.args.region:
            self.tester = EC2ops( credpath=self.args.credpath, region=self.args.region)
        else:
            self.tester = Eucaops( credpath=self.args.credpath, config_file=self.args.config,password=self.args.password)
        self.tester.poll_count = 120

        ### Add and authorize a group for the instance
        self.group = self.tester.add_group(group_name="group-" + str(time.time()))
        self.tester.authorize_group_by_name(group_name=self.group.name )
        self.tester.authorize_group_by_name(group_name=self.group.name, port=-1, protocol="icmp" )
        ### Generate a keypair for the instance
        self.keypair = self.tester.add_keypair( "keypair-" + str(time.time()))
        self.keypath = '%s/%s.pem' % (os.curdir, self.keypair.name)
        self.image = self.args.emi
        if not self.image:
            self.image = self.tester.get_emi(root_device_type="instance-store")
        self.address = None
        self.volume = None
        self.snapshot = None
        self.private_addressing = False
        zones = self.tester.ec2.get_all_zones()
        self.zone = random.choice(zones).name
        self.reservation = None

    def clean_method(self):
        ### Terminate the reservation if it is still up
        if self.reservation:
            self.assertTrue(self.tester.terminate_instances(self.reservation), "Unable to terminate instance(s)")

        ### DELETE group
        self.tester.delete_group(self.group)

        ### Delete keypair in cloud and from filesystem
        self.tester.delete_keypair(self.keypair)
        os.remove(self.keypath)

    def MyTest(self):
        """
        This case was developed to run through a series of basic instance tests.
             The tests are as follows:
                   - execute run_instances command
                   - make sure that public DNS name and private IP aren't the same
                       (This is for Managed/Managed-NOVLAN networking modes)
                   - test to see if instance is ping-able
                   - test to make sure that instance is accessible via ssh
                       (ssh into instance and run basic ls command)
             If any of these tests fail, the test case will error out, logging the results.
        """
        if not self.reservation:
            self.reservation = self.tester.run_instance(self.image, keypair=self.keypair.name, group=self.group.name)
        for instance in self.reservation.instances:
            self.assertTrue( self.tester.wait_for_reservation(self.reservation) ,'Instance did not go to running')
            self.assertNotEqual( instance.public_dns_name, instance.private_ip_address, 'Public and private IP are the same')
            self.assertTrue( self.tester.ping(instance.public_dns_name), 'Could not ping instance')
            self.assertFalse( instance.found("ls -1 /dev/" + instance.rootfs_device + "2",  "No such file or directory"),  'Did not find ephemeral storage at ' + instance.rootfs_device + "2")

if __name__ == "__main__":
    testcase = InstanceBasics()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list
    list = testcase.args.tests or [ "MyTest"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)


#! ../share/python_lib/matt_dev/bin/python
'''
Test Summary: 

-create a volume (do this first)
-run an instance (do this second, if this fails at least we know we could create a vol)

Usage Tests: 
-negative -attempt to attach a volume to an instance in a separate cluster. 
-attach a single volume to an instance in the zones given, write random data and calc md5 of volumes
-negative:attempt to delete the attached instance, should fail
-negative:attempt to attach an in-use volume, should fail
-attach a 2nd volume to an instance, write random date to vol and calc md5 of volumes
-reboot instance
-verify both volumes are attached after reboot of instance
-detach 1st volume
-create snapshot of detached volume
-create snapshot of attached volume

Multi-cluster portion...
-attempt to create a volume of each snapshot, if multi 1 in each cluster
-attempt to attach each volume to an instance verify md5s

Properties tests:
-create a volume of greater than prop size, should fail
-create a 2nd volume attempting to exceed the max aggregate size, should fail


Cleanup:
-remove all volumes, instance, and snapshots created during this test

'''
import types
import time
import os

from eucaops import Eucaops
from eutester import euinstance
from eutester.eutestcase import EutesterTestCase
from eucaops import ec2ops
from eutester.eutestcase import TestColor

class TestZone():
    def __init__(self, partition):
        self.partition = partition 
        self.name = partition
        self.instances = []
        self.volumes = []
        
    def __str__(self):
        return self.name

class EbsTestSuite(EutesterTestCase):
    
    tester = None
    zonelist = []
    snaps = []
    keypair = None
    group = None
    multicluster=False
    image = None
    
    def __init__(self, 
                 name=None,
                 args=None,
                 tester=None, 
                 zone=None, 
                 config_file='../input/2b_tested.lst', 
                 password="foobar",
                 user_data=None,
                 inst_pass=None,
                 credpath=None, 
                 volumes=None, 
                 keypair=None, 
                 group=None, 
                 emi=None,
                 waitconnect=30,
                 wait_on_progress=20,
                 root_device_type='instance-store',
                 vmtype='c1.medium',
                 eof=1):
        
        self.args = args
        self.setuptestcase(name)
        self.wait_on_progress = wait_on_progress
        if tester is None:
            self.tester = Eucaops( config_file=config_file,password=password,credpath=credpath)
        else:
            self.tester = tester
        self.tester.exit_on_fail = eof
    
        self.testlist =[]
        self.inst_pass=inst_pass
        if emi:
            self.image = self.tester.get_emi(emi=emi)
        else:
            self.image = self.tester.get_emi(root_device_type=root_device_type, not_location='windows')
        self.vmtype = vmtype
        self.zone = None
        self.waitconnect=int(waitconnect)
        self.zonelist = []
        self.user_data = user_data
        #create some zone objects and append them to the zonelist
        if self.zone:
            self.zone = TestZone(zone)
            self.zonelist.append(self.zone)
        else: 
            self.setup_testzones()
    
        #If the list of volumes passed in looks good, sort them into the zones
        if self.volumes_list_check(volumes):
            self.sort_volumes(volumes)
            
        #Setup our security group for later use
        if (group is not None):
            self.group = group
        else:
            group_name='EbsTestGroup'
            
            try:
                self.group = self.tester.add_group(group_name,fail_if_exists=False)
                self.tester.authorize_group_by_name(self.group.name)
                self.tester.authorize_group_by_name(self.group.name,protocol="icmp",port=-1)
            except Exception, e:  
                self.debug(self.tester.get_traceback())  
                raise Exception("Error when setting up group:"+str(group_name)+", Error:"+str(e))   
        
    
        #Setup the keypairs for later use
        if not self.inst_pass:
            try:
                if (keypair is not None):
                    self.keypair = keypair
                else:     
                    keys = self.tester.get_all_current_local_keys() 
                    if keys != []:
                        self.keypair = keys[0]
                    else:
                        self.keypair = keypair = self.tester.add_keypair('ebs_test_key-' + str(time.time()))
            except Exception, ke:
                raise Exception("Failed to find/create a keypair, error:" + str(ke))
        
        
    def setup_testzones(self):
        for zone in self.tester.get_zones():
                tzone = TestZone(zone)
                self.zonelist.append(tzone)
                self.multicluster=True
        if not self.zonelist:
            raise Exception("Could not discover an availability zone to perform tests in. Please specify zone")
    
    def volumes_list_check(self, volumes):
        #helper method to validate volumes for use as a list
        if (volumes is not None) and (volumes != []) and (not isinstance(volumes,basestring)):
            return True
        else:
            return False

    def instances_list_check(self, instances):
        #helper method to validate instances for use as a list
        if (instances is not None) and (instances != []) and (not isinstance(instances,basestring)):
            return True
        else:
            return False
        
    
    def sort_volumes(self, volumes):
        for vol in volumes:
            for zone in self.zonelist:
                if vol.zone == zone.name:
                    zone.volumes.append(vol)

        
    def create_vols_per_zone(self, zonelist=None, volsperzone=1, size=1, snapshot=None, timepergig=300):
        """
        Description:
                    Intention of this test is to verify creation of volume(s) per zone given.
                    Upon successful creation the volumes will be appended to a volumes list
                    for the zone it was created in. 
                    These volumes may be later used if in later ebstests suite tests. 
        """    
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        for testzone in zonelist:
            zone = testzone.name
            vols = self.tester.create_volumes(zone, size=size, count=volsperzone, snapshot=snapshot,timepergig=timepergig)
            testzone.volumes.extend(vols)
            self.debug('create_vols_per_zone created vols('+str(len(vols))+') zone:'+str(zone))
            
           
            
          
    def create_test_instances_for_zones(self, zonelist=None, image=None, keypair=None, username='root', inst_pass=None, group=None, vmtype=None,count=1):
        """
        Description:
                    Create an instance within each TestZone object in zonelist to help test ebs functionality.
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        if image is not None:
            if isinstance(image,types.StringTypes):
                image = self.tester.get_emi(emi=image)    
        else:
            image = self.image
        if group is None:
            group = self.group
        if keypair is None:
            keypair = self.keypair
        inst_pass = inst_pass or self.inst_pass
                
        vmtype = vmtype or self.vmtype
        if keypair:
            keyname = keypair.name
        else:
            keyname = None
            
        for testzone in zonelist:
            zone = testzone.name
            instances = self.tester.run_image( image=image,
                                                keypair=keyname,
                                                group=group,
                                                username=username,
                                                password=inst_pass,
                                                user_data=self.user_data,
                                                type=vmtype,
                                                zone=zone,
                                                min=count,
                                                max=count)

            for inst in instances:
                testzone.instances.append(inst)
            self.debug('Created instance: ' + str(inst.id)+" in zone:"+str(zone))
        #self.endsuccess()
    
    def terminate_test_instances_for_zones(self, zonelist=None, timeout=360):
        if zonelist is None:
            zonelist = self.zonelist
        for zone in zonelist:
            for instance in zone.instances:
                self.tester.terminate_single_instance(instance, timeout)
                zone.instances.remove(instance)
                
    def terminate_instances_in_zones_verify_volume_detach(self,zonelist=None,timeout=360):
        """
        Description:
                  Iterates over all instances in this testcase's zonelist attempts to terminate the instances,
                  and verify the attached volumes go to available after the instances are terminated. 
        """
        instance = euinstance.EuInstance()
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        for zone in zonelist:
            for instance in zone.instances:
                instance.terminate_and_verify(verify_vols=True,timeout=timeout)
                zone.instances.remove(instance)

    def negative_attach_in_use_volume_in_zones(self,zonelist=None,timeout=360):
        """
        Description:
                    Iterates though zones and attempts to attach already attached volumes to instances within each zone.  
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        instance = euinstance.EuInstance()
        for zone in zonelist:
            tested = 0
            for volume in zone.volumes:
                volume.update()
                if (volume.status == "in-use"):
                    tested += 1
                    for instance in zone.instances:
                        try:
                            #This should fail
                            instance.attach_euvolume(volume,timeout=timeout)
                        except Exception, e:
                            #If it failed were good
                            self.debug("negative_attach_in_use_volume_in_zones Passed. Could not attach in-use volume")
                            #self.endsuccess()
                            pass
                        else:
                            #The operation did fail, but this test did
                            raise Exception("negative_attach_in_use_volume_in_zones failed volume attached")
            if not tested:
                raise Exception("No attached volumes found to test against")
                
    
    def attach_all_avail_vols_to_instances_in_zones(self, zonelist=None, timeout=360, overwrite=False):
        """
        Description:
                    Iterates though zones and attempts to attach volumes to an instance within each zone.  
                    
        :parram zonelist: list of zones to include in test
        :param timeout: timeout used for attach volume method
        :param overwrite: boolean to indicate whether a non-zero filled volume should have new unique data prepended for md5sum. 
                          This should be used when zero fill volume property is not in use upon volume first attach. 
                          It should not be used after the 1st attach and volume has been converted to a euvolume within this test. 
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("attach_all_avail_vols_to_instances_in_zones: Zonelist is empty")
        for zone in zonelist:
            if not zone.volumes:
                raise Exception('attach_all_avail_vols_to_instances_in_zones: Zone.volumes is empty')
            if not zone.instances:
                raise Exception('attach_all_avail_vols_to_instances_in_zones: Instance list is empty')
            i=0
            for volume in zone.volumes:
                volume.update()
                if (volume.status == "available"):
                        if i > (len(zone.instances)-1):
                            i = 0
                        self.debug("Attempting to attach to "+str(i)+"/"+str(len(zone.instances))+" instances in zone:"+str(zone))
                        instance = zone.instances[i]
                        try:
                            instance.attach_euvolume(volume,timeout=timeout, overwrite=overwrite)
                        except ec2ops.VolumeStateException, vse:
                            self.status("This is a temp work around for testing, this is to avoid bug euca-5297"+str(vse),
                                        testcolor=TestColor.get_canned_color('failred'))
                            time.sleep(10)
                            self.debug('Monitoring volume post VolumeStateException...')
                            volume.eutest_attached_status = None
                            self.tester.monitor_euvolumes_to_status([volume],status='in-use',attached_status='attached',timeout=60)
                        except Exception, e:
                            self.debug("attach_all_vols_to_instances_in_zones failed to attach volume")
                            raise e
                        i += 1
            
                    
                        
    def negative_delete_attached_volumes_in_zones(self,zonelist=None, timeout=60):
        """
        Description:
                    Negative test case. Attempts to delete attached volumes for each euinstace
                    in each zone per zone list provided. Confirms that volumes did NOT delete while in use/attached.            
        """
        #instance = euinstance.EuInstance()
        #volume = euvolume.EuVolume()
      
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        
        for zone in zonelist:
            if not zone.instances:
                raise Exception("No Instances in zone:"+str(zone.name))
            for instance in zone.instances:
                #resync instance volume state first
                self.debug('syncing volumes for instance:'+str(instance.id))
                badvols = instance.get_unsynced_volumes() 
                if (badvols is not None) and (badvols != []):
                    self.debug("negative_delete_attached_volumes_in_zones, failed")
                    try:
                        errmsg=""
                        for badvol in badvols:
                            errmsg=errmsg+str(badvol.id)+", "
                    except:pass
                    raise Exception("("+str(instance.id)+") Unsync'd volumes found:"+errmsg)
                #Attempt to delete volumes, confirm this operation does not succeed
                if not instance.attached_vols:
                    raise Exception("No attached volumes found for test")
                for volume in instance.attached_vols:
                    try:
                        volume.delete()
                    except: 
                        self.debug("Success- could not delete attached volume:"+str(volume.id))
                    else:
                        volume.update()
                        if (volume.status == "deleted"):
                            self.debug("negative_delete_attached_volumes_in_zones, failed:"+str(volume.id))
                            raise Exception("Was able to delete attached volume:"+str(volume.id))
                   
                        
    def reboot_instances_in_zone_verify_volumes(self,zonelist=None,waitconnect=30, timeout=360):
        """
        Description:
                    Attempts to iterate through each instance in each zone and reboot the instance(s). 
                    Attempts to verify the attached volume state post reboot. 
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        instance = euinstance.EuInstance()
        for zone in zonelist:
            if not zone.instances:
                raise Exception("No instances in zone:"+str(zone.name))
            for instance in zone.instances:
                instance.reboot_instance_and_verify(waitconnect=waitconnect, timeout=timeout, checkvolstatus=True)
        #self.endsuccess()
        
    def detach_volumes_in_zones(self,zonelist=None, timeout=360, volcount=1, eof=False):
        """
        Description:
                    Attempts to detach volcount volumes from each instance in the provided zonelist. 
                    If volcount is None or 0, will attempt to detach all volumes from all instances. 
                    Attempts to verify detached volume state on both the cloud and the guest
                    by default will attempt to detach a single volume from each instance
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        #instance = euinstance.EuInstance()
        for zone in zonelist:
            if not zone.instances:
                raise Exception("No instances in zone:"+str(zone.name))
            for instance in zone.instances:
                vc=0
                badvols = instance.get_unsynced_volumes() 
                if (badvols is not None) and (badvols != []):
                    self.debug("failed")
                    errlist=[]
                    for badvol in badvols:
                        errlist.append(str(badvol.id))
                    raise Exception("Unsync volumes found on:"+str(instance.id)+"\n"+" ".join(errlist))
                errmsg=""
                for volume in instance.attached_vols:
                    #detach number of volumes equal to volcount
                    if volcount and vc >= volcount:
                        break
                    else:
                        vc += 1
                        try:
                            instance.detach_euvolume(volume, timeout=timeout)
                        except Exception, e: 
                            self.debug("fail. Could not detach Volume:"+str(volume.id)+"from instance:"+str(instance.id))
                            if eof:
                                raise e
                            else:
                                errmsg += "\nCould not detach Volume:"+str(volume.id)+"from instance:"+str(instance.id)+",err:"+str(e)
                            
        if errmsg:
            raise Exception(errmsg)
        
    def detach_all_volumes_from_stopped_instances_in_zones(self,zonelist=None, timeout=360):
        """
        Description:
                    Attempts to detach volumes from instances while in the stopped state and
                    verify volumes are detached, and upon instance start verify that both guest
                    and cloud states are correct.
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        for zone in zonelist:
            if not zone.instances:
                raise Exception("No instances in zone:"+str(zone.name))
            for instance in zone.instances:
                if instance.block_device_mapping != 'ebs':
                    continue
                if not instance.attached_vols:
                    raise Exception('detach_all_volumes_from_stopped_instances_in_zones: No attached volumes for:'+str(instance.id))
                instance.stop_instance_and_verify()
                for vol in instance.attached_vols:
                    instance.detach_euvolume(vol, waitfordev=False)
                instance.start_instance_and_verify(checkvolstatus=True)
        #self.endsuccess()
    
    def delete_volumes_in_zones(self, zonelist=None, timeout=60):
        """
        Description:
                    Attempts to iterate over each zone and delete all test volumes.
        
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        for zone in zonelist:
            for volume in zone.volumes:
                start = time.time()
                elapsed = 0
                volume.delete()
                while (volume.status != "deleted") and (elapsed < timeout):
                    volume.update()
                    elapsed = int(time.time()-start)
                if volume.status != "deleted":
                    self.debug("failed to delete volume:"+str(volume.id))
                else:
                    zone.volumes.remove(volume)
        #self.endsuccess()
        
        
    def delete_snapshots_in_zones(self, zonelist=None,snaplist=None, timeout=300):
        """
        Description:
                    Attempts to iterate through zonelist, and delete all snapshots 
                    within that zone
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        if snaplist is None:
            snaplist = self.snaps
        for zone in zonelist:
            for snap in snaplist:
                if snap.eutest_volume_zone == zone:
                    self.tester.delete_snapshot(snap, timeout=timeout)
                    snaplist.remove(snap)
        #self.endsuccess()
        
                
        
        
    def create_snapshots_all_vols_in_zone(self, zonelist=None, volstate="all", wait_on_progress=None):
        """
        Description:
                    Attempts to iterate through each zone in zonelist, and create a snapshot from each volume
                    in the zone's volume list who's state matches volstate
                    
        """
        wait_on_progress = wait_on_progress or self.wait_on_progress
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        for zone in zonelist:
            if not zone.volumes:
                raise Exception("No volumes in zone:"+str(zone.name))
            for volume in zone.volumes:
                volume.update()
                if volstate == "all" or volume.status == volstate:
                    self.snaps.append(self.tester.create_snapshot_from_volume(volume, description="ebstest", wait_on_progress=wait_on_progress))
        #self.endsuccess()
        
        
    def create_vols_from_snap_in_same_zone(self, zonelist=None,timepergig=300):
        """
        Description:
                    Attempts to create a volume from each snapshot contained in each zone's list of snapshots.
                    This test attempts to create volumes from snapshots who's original volume is also in this
                    zone. 
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        for zone in zonelist:
            if not self.snaps:
                raise Exception("Create_Vols_from_snap_in_same_zone error: No snapshots available for test")
            zonesnaps = self.get_snaps_from_zone(self.snaps, zone)
            if not zonesnaps:
                raise Exception("No Snapshots from this test found in zone:"+str(zone))
            for snap in zonesnaps:
                self.debug("Creating volume from snap:"+str(snap.id))
                newvol = self.tester.create_volume(zone.name, size=0, snapshot=snap,timepergig=timepergig)
                zone.volumes.append(newvol)
                snap.eutest_volumes.append(newvol)
        #self.endsuccess()
        
    def get_snaps_from_zone(self,snaplist, zone):
        retlist = []
        for snap in snaplist:
            self.debug(snap.id+", zone:"+str(zone)+" snap.eutest_volume_zone:"+str(snap.eutest_volume_zone))
            if str(snap.eutest_volume_zone).strip() == str(zone).strip():
                self.debug('Adding snap to retlist:'+str(snap.id))
                retlist.append(snap)
        return retlist
        
    def attach_new_vols_from_snap_verify_md5(self,zonelist=None, timeout=360,timepergig=360):
        """
        Description:
                    Attempts to attach volumes which were created from snapshots and are not in use. 
                    Iterates over test instances in zones for attaching the test volumes. 
                    After verifying the volume is attached and reported as so by cloud and guest, 
                    this test will attempt to compare the md5 sum of the volume to the md5 contained in 
                    the snapshot which represents the md5 of the original volume. 
                    This test accepts a timepergig value which is used to guesstimate a reasobale timeout while
                    waiting for the md5 operation to be executed. 
        """
        
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("attach_new_vols_from_snap_verify_md5: Zonelist is empty")
        for zone in zonelist:
            self.debug("checking zone:"+zone.name)
            
            if not self.snaps:
                raise Exception('attach_new_vols_from_snap_verify_md5: self.snaps is None')
            for snap in self.snaps:
                self.debug("Checking volumes associated with snap:"+snap.id)
                if not snap.eutest_volumes:
                    raise Exception('attach_new_vols_from_snap_verify_md5: snap '+str(snap.id)+" eutest_volumes is None")
                i = 0
                for vol in snap.eutest_volumes:
                    self.debug("Checking volume:"+vol.id+" status:"+vol.status)
                    if (vol.zone == zone.name) and (vol.status == "available"):
                        if i > len(zone.instances)-1:
                            i = 0
                        instance = zone.instances[i]
                        try:
                            instance.attach_euvolume(vol, timeout=timeout)
                        except ec2ops.VolumeStateException, vse:
                            self.status("This is a temp work around for testing, this is to avoid bug euca-5297"+str(vse),
                                        testcolor=TestColor.get_canned_color('failred'))
                            time.sleep(10)
                            self.debug('Monitoring volume post VolumeStateException...')
                            vol.eutest_attached_status = None
                            self.tester.monitor_euvolumes_to_status([vol],status='in-use',attached_status='attached',timeout=60)
                        except Exception, e:
                            self.debug("Failed to attach volume: " + str(vol.id) + "to instance:" + str())
                            raise e
                        instance.md5_attached_euvolume(vol, timepergig=timepergig)
                        if vol.md5 != snap.eutest_volume_md5:
                            self.debug("snap:"+str(snap.eutest_volume_md5)+" vs vol:"+str(vol.md5))
                            self.debug("Volume:"+str(vol.id)+" MD5:"+str(vol.md5)+" != Snap:"+str(snap.id)+" MD5:"+str(snap.eutest_volume_md5))
                            raise Exception("Volume:"+str(vol.id)+" MD5:"+str(vol.md5)+" != Snap:"+str(snap.id)+" MD5:"+str(snap.eutest_volume_md5))
                        self.debug("Successfully verified volume:"+str(vol.id)+" to snapshot:"+str(snap.id))
                        i += 1
        #self.endsuccess()
        
    def create_vols_from_snap_in_different_zone(self,zonelist=None, timepergig=300):
        """
        Description:         
                    Attempts to create a volume from each snapshot contained in each zone's list of snapshots.
                    This test attempts to create volumes from snapshots who's original volume is "NOT" in this 
                    same zone
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        for zone in zonelist:
            for snap in self.snaps:
                if snap.eutest_volume_zone != zone:
                    newvol = self.tester.create_volume(zone.name,size=0, snapshot=snap, timepergig=timepergig)
                    zone.volumes.append(newvol)
                    snap.eutest_volumes.append(newvol)
        #self.endsuccess()
        
    
    def consecutive_snapshot_to_vol_verify_md5s(self,
                                                zonelist=None, 
                                                count=5, 
                                                volmaxsize=1, 
                                                delay=0, 
                                                tpg=300,
                                                delete_to=120,
                                                poll_progress=60,
                                                attach_timeout=360):
        """
        Description:
                   Attempts to create a 'count' number of snapshots consecutively with a delay of 'delay'
                   between each creation attempt. If snapshot % progress does not increase within 'wait_on_progress'
                   10 second poll intervals, test will fail. IF snapshots are successfully created. Then they will
                   each have a volume created, and attached to an instance to verify the md5 against the original volume. 
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        zone = TestZone
        #vol = euvolume.EuVolume
        instance = euinstance.EuInstance
        for zone in zonelist:
            snaps =[]
            vols = []
            createdvols = []
            self.status('STARTING ZONE:'+str(zone.name))
            if not zone.instances or not zone.volumes:
                raise Exception("Zone "+str(zone.name)+", did not have at least 1 volume and 1 instance to run test")
            instance = zone.instances[0]
            for vol in zone.volumes:
                if vol.size <= volmaxsize:
                    break
            if vol.size > volmaxsize:
                raise Exception("Could not find volume in zone "+str(zone.name)+" <= volmaxsize of:"+str(volmaxsize))
            self.status("Attempting to create "+str(count)+" snapshots in zone:"+str(zone.name)+"...")
            snaps = self.tester.create_snapshots(vol, count=count, delay=delay, wait_on_progress=poll_progress)
            self.debug('Finished creating '+str(count)+' snapshots in zone:'+str(zone.name)+', now creating vols from them')
            try:
                for snap in snaps:
                    createdvols.extend(self.tester.create_volumes(zone,snapshot=snap,timepergig=tpg, monitor_to_state=False))
                vols.extend(self.tester.monitor_created_euvolumes_to_state(createdvols, timepergig=tpg))
                self.tester.print_euvolume_list(vols)
                self.status("Attempting to attach new vols from new snapshots to instance:"+str(instance.id)+" to verify md5s...")
                for newvol in vols:
                    try:
                        instance.attach_euvolume(newvol,timeout=attach_timeout)
                    except ec2ops.VolumeStateException, vse:
                        self.status("This is a temp work around for testing, this is to avoid bug euca-5297"+str(vse),
                                    testcolor=TestColor.get_canned_color('failred'))
                        time.sleep(10)
                        self.debug('Monitoring volume post VolumeStateException...')
                        newvol.eutest_attached_status = None
                        self.tester.monitor_euvolumes_to_status([newvol],status='in-use',attached_status='attached',timeout=60)
                    except Exception, e:
                        self.debug("Failed to attach volume: " + str(newvol.id) + "to instance:" + str())
                        raise e
                    if vol.md5 != newvol.md5:
                        raise Exception("New volume's md5:"+str(newvol.md5)+" !=  original volume md5:"+str(vol.md5))
                    else:
                        self.debug("Success. New volume:"+str(newvol.id)+"'s md5:"+str(newvol.md5)+" ==  original volume:"+str(vol.id)+"'s md5:"+str(vol.md5))
                    instance.detach_euvolume(newvol)
            finally:
                self.debug("Attempting to cleanup/delete snapshots and volumes from this test...")
                #add snapshots to global list for cleanup later
                self.snaps.extend(snaps)
                for avol in instance.attached_vols:
                    if avol in vols:
                        instance.detach_euvolume(avol, waitfordev=False)
                delfail = None  
                for vol in vols:
                    try:
                        self.tester.delete_volume(vol,timeout=delete_to)
                    except Exception, e:
                        delfail = str(vol.id)+" failed to delete, err:"+str(e)
                if delfail:
                    raise Exception(delfail)
               
            
    def concurrent_consecutive_volumes_from_snap_verify_md5(self, 
                                                            zonelist=None, 
                                                            snap=None, 
                                                            count=5, 
                                                            volmaxsize=1, 
                                                            delay=0, 
                                                            tpg=300,
                                                            delete_to=120,
                                                            poll_progress=60,
                                                            attach_timeout=360):
        """
        Description:
                   Attempts to create a 'count' number of volumes from a given snapshot consecutively with a delay of 'delay'
                   between each creation attempt. Waits for volumes to become available. Attempts to attach each volume and
                   verify it's md5 sum matches the original volumes. If multiple zones are specified will try in both zones at the same time. 
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        zone = TestZone
        #vol = euvolume.EuVolume
        instance = euinstance.EuInstance
        if not snap:
            for snap in self.snaps:
                if snap.volume_size <= volmaxsize:
                    break
            if not snap or snap.volume_size > volmaxsize:
                raise Exception("Could not find  snapshot <= volmaxsize of:"+str(volmaxsize))
        if snap.eutest_volume_md5:
            origmd5 = snap.eutest_volume_md5 
        else: 
            raise Exception('Snapshot must be created from a previously attached euvolume in order to derive md5')
        self.debug('Using Snapshot:'+str(snap.id))
        for zone in zonelist:
            if not zone.instances:
                raise Exception("Zone "+str(zone.name)+", did not have at least 1 instance to run test")
        vols = []
        instances = []
        try:
            for zone in zonelist:
                self.status('STARTING ZONE:'+str(zone.name))
                #Do not set monitor flag in order to quickly request count number of consecutive vols in each zone
                vols.extend(self.tester.create_volumes(zone,snapshot=snap, count=count, monitor_to_state=None, timepergig=tpg))
            vols = self.tester.monitor_created_euvolumes_to_state(vols,timepergig=tpg)
            self.tester.print_euvolume_list(vols)
            for zone in zonelist:
                instance = zone.instances[0]
                instances.append(instance)
                self.status("Attempting to attach new vols from new snapshots to instance:"+str(instance.id)+" to verify md5s...")      
                for newvol in vols:
                    if newvol.zone == zone.name:
                        try:
                            instance.attach_euvolume(newvol, timeout=attach_timeout)
                        except ec2ops.VolumeStateException, vse:
                            self.status("This is a temp work around for testing, this is to avoid bug euca-5297"+str(vse),
                                    testcolor=TestColor.get_canned_color('failred'))
                            time.sleep(10)
                            self.debug('Monitoring volume post VolumeStateException...')
                            newvol.eutest_attached_status = None
                            self.tester.monitor_euvolumes_to_status([newvol],status='in-use',attached_status='attached',timeout=60)
                        except Exception, e:
                            self.debug("Failed to attach volume: " + str(newvol.id) + "to instance:" + str())
                            raise e
                        #Compare MD5 sum to original volume
                        if str(origmd5).rstrip().lstrip() != str(newvol.md5).rstrip().lstrip():
                            raise Exception("New volume's md5:'"+str(newvol.md5)+"' !=  original volume md5:'"+str(origmd5)+"'")
                        else:
                            self.debug("Success. New volume:"+str(newvol.id)+"'s md5:"+str(newvol.md5)+" ==  original volume:"+str(snap.volume_id)+"'s md5:"+str(origmd5))
                        instance.detach_euvolume(newvol)
        
        finally:
            self.debug("Attempting to cleanup/delete snapshots and volumes from this test...")
            for instance in instances:
                for avol in instance.attached_vols:
                    if avol in vols:
                        instance.detach_euvolume(avol)
            self.tester.print_euvolume_list(vols)
            delfail = None
            for vol in vols:
                try:
                    self.tester.delete_volume(vol,timeout=delete_to)
                except Exception, e:
                    delfail = str(vol.id)+" failed to delete, err:"+str(e)
            if delfail:
                raise Exception(delfail)
            
                   
    def test_multi_node(self,run=True, count=10, nodecount=2):
        testlist = [] 
        #create 4 volumes per zone
        testlist.append(self.create_testunit_from_method(self.create_vols_per_zone, volsperzone=(2*nodecount), eof=True))
        #launch instances to interact with ebs volumes per zone
        testlist.append(self.create_testunit_from_method(self.create_test_instances_for_zones, count=nodecount, eof=True))
        
        for x in xrange(0,count):
            #attach first round of volumes
            testlist.append(self.create_testunit_from_method(self.attach_all_avail_vols_to_instances_in_zones, 
                                                             overwrite=True, 
                                                             eof=True))
            #detach 1 volume leave the 2nd attached
            testlist.append(self.create_testunit_from_method(self.detach_volumes_in_zones))
        
            
        #terminate each instance and verify that any attached volumes return to available state
        testlist.append(self.create_testunit_from_method(self.terminate_instances_in_zones_verify_volume_detach))
        
        if run:
            self.run_test_case_list(testlist)
        else:
            return testlist
    
    def test_consecutive_concurrent(self,run=True, count=5, delay=0, tpg=300, poll_progress=60, delete_to=120, snap_attached=False):
        testlist = [] 
        #create 1 volume per zone
        testlist.append(self.create_testunit_from_method(self.create_vols_per_zone, volsperzone=1, eof=True))
        #launch an instances to interact with ebs volumes per zone
        testlist.append(self.create_testunit_from_method(self.create_test_instances_for_zones, eof=True))
        #attach first round of volumes
        testlist.append(self.create_testunit_from_method(self.attach_all_avail_vols_to_instances_in_zones, 
                                                         overwrite=True, 
                                                         eof=True))
        if not snap_attached:
            #detach 1 volume 
            testlist.append(self.create_testunit_from_method(self.detach_volumes_in_zones))
        
        #Attempt to create multiple snapshots quickly then volumes from thos snaps and verify the md5 against original volume's
        testlist.append(self.create_testunit_from_method(self.consecutive_snapshot_to_vol_verify_md5s, 
                                                         count=count, 
                                                         delay=delay, 
                                                         tpg=tpg,
                                                         delete_to=delete_to,
                                                         poll_progress=poll_progress))
        #attempt to create volumes from snaps, attach and verify md5 in same zone it was created in
        testlist.append(self.create_testunit_from_method(self.create_snapshots_all_vols_in_zone))
        #Attempt to create multiple consecutive volumes from a single snapshot, will attempt concurrent tests accross multiple zones if in multi zone test
        testlist.append(self.create_testunit_from_method(self.concurrent_consecutive_volumes_from_snap_verify_md5,
                                                         count=count, 
                                                         delay=delay, 
                                                         tpg=tpg,
                                                         delete_to=delete_to))
        #terminate each instance and verify that any attached volumes return to available state
        testlist.append(self.create_testunit_from_method(self.terminate_instances_in_zones_verify_volume_detach))
        if run:
            self.run_test_case_list(testlist)
        else:
            return testlist


    def expand_volume_size(self, zonelist=None, volsperzone=1, size=1):
        """
        Description:
                    Intention of this test is to verify creation of volume(s) from a snapshot and expanding
                    the size of the volume
        """
        zonelist = zonelist or self.zonelist
        if not zonelist:
            raise Exception("Zone list was empty")
        for testzone in zonelist:
            vols = self.tester.create_volumes(testzone, size=size, count=volsperzone)
            testzone.volumes.extend(vols)
            snapshots = []
            for volume in vols:
                snapshots.append(self.tester.create_snapshot_from_volume(volume))
            larger_volumes = []
            for snaphot in snapshots:
                larger_volumes.append(self.tester.create_volume(testzone, snapshot=snaphot, size=size+1))
            for volume in larger_volumes:
                assert volume.size > size


    def ebs_basic_test_suite(self, run=True):  
        testlist = [] 
        #create first round of volumes
        testlist.append(self.create_testunit_from_method(self.create_vols_per_zone, eof=True))
        #create volumes that have their sizes expanded from their original snapshots
        testlist.append(self.create_testunit_from_method(self.expand_volume_size, eof=True))
        #launch instances to interact with ebs volumes
        testlist.append(self.create_testunit_from_method(self.create_test_instances_for_zones, eof=True))
        #attach first round of volumes
        testlist.append(self.create_testunit_from_method(self.attach_all_avail_vols_to_instances_in_zones, overwrite=True, eof=True))
        #attempt to delete attached volumes, should not be able to
        testlist.append(self.create_testunit_from_method(self.negative_delete_attached_volumes_in_zones))
        #attempt to attach a volume which is already attached, should not be able to
        testlist.append(self.create_testunit_from_method(self.negative_attach_in_use_volume_in_zones))
        #create second round of volumes
        testlist.append(self.create_testunit_from_method(self.create_vols_per_zone))
        #attach second round of volumes
        testlist.append(self.create_testunit_from_method(self.attach_all_avail_vols_to_instances_in_zones, overwrite=True))
        #reboot instances and confirm volumes remain attached
        testlist.append(self.create_testunit_from_method(self.reboot_instances_in_zone_verify_volumes, waitconnect=self.waitconnect))
        #detach 1 volume leave the 2nd attached
        testlist.append(self.create_testunit_from_method(self.detach_volumes_in_zones))
        #attempt to create volumes from snaps, attach and verify md5 in same zone it was created in
        testlist.append(self.create_testunit_from_method(self.create_snapshots_all_vols_in_zone))
        #attempt to create volumes of each snap within the same zone they were originally created in
        testlist.append(self.create_testunit_from_method(self.create_vols_from_snap_in_same_zone))
        #attempt to verify integrity of the volumes  by attaching to instance and checking md5 against original
        testlist.append(self.create_testunit_from_method(self.attach_new_vols_from_snap_verify_md5))  
        if (len(self.zonelist) > 1 ):
            #attempt to create volumes from     s, attach and verify md5 in a different zone than it was created in        
            testlist.append(self.create_testunit_from_method(self.create_vols_from_snap_in_different_zone))
            #verify the integrity of the new volumes by attaching to instance and checking md5 against original
            testlist.append(self.create_testunit_from_method(self.attach_new_vols_from_snap_verify_md5))
        #'IF' a bfebs instance was used, confirm attached volumes can be detached while in stopped state
        testlist.append(self.create_testunit_from_method(self.detach_all_volumes_from_stopped_instances_in_zones))
        #terminate each instance and verify that any attached volumes return to available state
        testlist.append(self.create_testunit_from_method(self.terminate_instances_in_zones_verify_volume_detach))
        if run:
            self.run_test_case_list(testlist)
        else:
            return testlist
        
                
    
    def restart_clc_makevol(self, zonelist=None):
        '''
        Test start/stop cloud service recovery as it pertains to storage
        ''' 
        if zonelist is None:
            zonelist = self.zonelist
    
        clc = self.tester.service_manager.get_enabled_clc()
        clc.stop()
        clc.start()
        self.tester.service_manager.wait_for_service(clc)
        for zone in zonelist:
            start = time.time()
            elapsed = 0
            sc = None
            while (sc is None) and (elapsed < 360):
                self.debug("waiting for sc in zone:"+str(zone.name)+" elapsed:"+str(elapsed))
                elapsed = int(time.time()-start)
                sc = zone.partition.get_enabled_sc()
                time.sleep(5)
        #wait a bit before attempting a volume
        if sc is None:
            raise Exception("Elapsed:"+str(elapsed)+"Couldn't find enabled sc for zone"+str(zone.name))
        self.debug("SC gone to enabled after "+str(elapsed)+" seconds")
        time.sleep(30)
        self.create_vols_per_zone()
        self.debug("Done creating volumes, now delete em...")
        for zone in zonelist:
            self.debug("Deleting volumes in zone:"+str(zone.name))
            for volume in zone.volumes:
                self.debug("Deleting volume:"+str(volume.id))
                volume.delete()
                start = time.time()
                elapsed = 0
                while ( volume.status == "deleted") and (elapsed < 100 ):
                    volume.update()
                    elapsed = int(time.time()- start)
                    self.debug("Waiting for volume:"+str(volume)+" to delete. elapsed:"+str(elapsed))
                    time.sleep(5)
                if ( volume.status == "deleted") :
                    raise Exception("Volume:"+str(volume)+" failed to delete. state:"+str(volume.status))
        
        
    def spin_restart(self, count=1000):
        '''
        Churn test wrapping cloud service start/stop storage tests
        '''
        for x in xrange(0,count):
            self.startmsg("test attempt("+str(x)+")")
            self.restart_clc_makevol()
            self.endsuccess("test attempt("+str(x)+")")
    
    
    
    def test_max_volume_size_property(self, volumes=None, maxsize=1, zones=None):
        if zones is None or zones == []:
            zones = self.zones
    def clean_method(self):
        """
        Definition:
        Attempts to clean up test artifacts created during this test
        """

        self.clean_created_resources(zonelist=self.zonelist, timeout=360)
    
    def clean_created_resources(self, zonelist=None, timeout=360):
        """
        Definition:
        Attempts to clean up test artifacts created during this test
        """
        self.tester.cleanup_artifacts()
        #self.terminate_test_instances_for_zones(zonelist=zonelist, timeout=timeout)
        #self.delete_volumes_in_zones(zonelist=zonelist, timeout=timeout)
        #self.delete_snapshots_in_zones(zonelist=zonelist,  timeout=timeout)
   
            
    
if __name__ == "__main__":
    ## If given command line arguments, use them as test names to launch

    tc = EutesterTestCase()

    tc.setup_parser(testname='ebstestsuite.py', description='collection of ebs related tests', testlist=False)
    tc.parser.add_argument('--inst_pass', 
                        help="Instance password for ssh session if not key enabled", default=None)
    
    args = tc.get_args()
    #if file was not provided or is not found
    if not os.path.exists(args.config):
        print "Error: Mandatory Config File '"+str(args.config)+"' not found."
        tc.parser.print_help()
        exit(1)
    #ebssuite = EbsTestSuite(zone=args.zone, config_file= args.config, password=args.password,credpath=args.credpath, keypair=args.keypair, group=args.group, image=args.emi)
    ebssuite = tc.do_with_args(EbsTestSuite)
    kbtime=time.time()
    try:
       list = ebssuite.ebs_basic_test_suite(run=False)
       tc.run_test_case_list(list)
    except KeyboardInterrupt:
        ebssuite.debug("Caught keyboard interrupt...")
        if ((time.time()-kbtime) < 2):
            ebssuite.clean_created_resources()
            ebssuite.debug("Caught 2 keyboard interupts within 2 seconds, exiting test")
            ebssuite.clean_created_resources()
            tc.print_test_list_results(list)
            raise
        else:          
            tc.print_test_list_results(list)
            kbtime=time.time()
            pass     
    except Exception, e:
        raise e
        exit(1)
    finally:
        tc.print_test_list_results(list)
    exit(0)
        



  



#!/usr/bin/python
from eucaops import Eucaops
import argparse
if __name__ == "__main__":
    ## If given command line arguments, use them as test names to launch
    parser = argparse.ArgumentParser(description='Get credentials for a cloud')
    parser.add_argument('--config', default="../input/2b_tested.lst")
    parser.add_argument("--password",default='foobar')
    args = parser.parse_args()
    tester = Eucaops(config_file=args.config, password=args.password)       



#!/usr/bin/python
import time
import re
import boto

from testcases.cloud_user.instances.instancetest import InstanceBasics
from testcases.cloud_user.s3.bucket_tests import BucketTestSuite
from eucaops import Eucaops
import os
import random

class HAtests(InstanceBasics, BucketTestSuite):
    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.get_args()
        if not boto.config.has_section('Boto'):
            boto.config.add_section('Boto')
            boto.config.set('Boto', 'num_retries', '1')
            boto.config.set('Boto', 'http_socket_timeout', '20')
        self.tester = Eucaops( config_file=self.args.config_file, password=self.args.password)
        self.tester.ec2.connection.timeout = 30
        self.servman = self.tester.service_manager
        self.instance_timeout = 120
        ### Add and authorize a group for the instance
        self.start_time = str(int(time.time()))
        try:
            self.group = self.tester.add_group(group_name="group-" + self.start_time )
            self.tester.authorize_group_by_name(group_name=self.group.name )
            self.tester.authorize_group_by_name(group_name=self.group.name, port=-1, protocol="icmp" )
            ### Generate a keypair for the instance
            self.keypair = self.tester.add_keypair( "keypair-" + self.start_time)
            self.keypath = os.curdir + "/" + self.keypair.name + ".pem"
            if self.args.emi:
                self.image = self.tester.get_emi(self.args.emi)
            else:
                self.image = self.tester.get_emi(root_device_type="instance-store")
            self.reservation = None
            self.private_addressing = False
            self.bucket_prefix = "buckettestsuite-" + self.start_time + "-"
            self.test_user_id = self.tester.s3.get_canonical_user_id()
            zones = self.tester.ec2.get_all_zones()
            self.zone = random.choice(zones).name

            self.tester.clc = self.tester.service_manager.get_enabled_clc().machine
            self.version = self.tester.clc.sys("cat " + self.tester.eucapath + "/etc/eucalyptus/eucalyptus-version")[0]
            ### Create standing resources that will be checked after all failures
            ### Instance, volume, buckets
            ###
            self.standing_reservation = self.tester.run_instance(image=self.image ,keypair=self.keypair.name,group=self.group.name, zone=self.zone)
            self.volume = self.tester.create_volume(self.zone)
            self.device = self.standing_reservation.instances[0].attach_volume(self.volume)
            for instance in self.standing_reservation.instances:
                instance.sys("echo " + instance.id  + " > " + self.device)
            self.standing_bucket_name = "failover-bucket-" + self.start_time
            self.standing_bucket = self.tester.create_bucket(self.standing_bucket_name)
            self.standing_key_name = "failover-key-" + self.start_time
            self.standing_key = self.tester.upload_object(self.standing_bucket_name, self.standing_key_name)
            self.standing_key = self.tester.get_objects_by_prefix(self.standing_bucket_name, self.standing_key_name)
            self.run_instance_params = {'image': self.image, 'keypair': self.keypair.name, 'group': self.group.name,
                                        'zone': self.zone, 'timeout': self.instance_timeout}
        except Exception, e:
            self.clean_method()
            raise Exception("Init for testcase failed. Reason: " + str(e))

    def clean_method(self):
        if hasattr(self,"standing_reservation") and self.standing_reservation:
            self.tester.terminate_instances(self.standing_reservation)
        if hasattr(self,"reservation") and self.reservation:
            self.tester.terminate_instances(self.reservation)
        if hasattr(self,"volume") and self.volume:
            self.tester.delete_volume(self.volume)
        self.servman.start_all()
        self.servman.all_services_operational()

    def clean_testcase(self):
        if self.reservation:
            self.tester.terminate_instances(self.reservation)
        self.reservation = None

    def run_testcase(self, testcase_callback, **kwargs):
        poll_count = 20
        poll_interval = 20
        while (poll_count > 0):
            try:
                testcase_callback(**kwargs)
                break
            except Exception, e:
                self.tester.debug("Attempt failed due to: " + str(e)  + "\nRetrying testcase in " + str(poll_interval) )
            finally:
                self.clean_testcase()
            self.tester.sleep(poll_interval)
            poll_count = poll_count - 1  
        if poll_count is 0:
            self.fail("Could not run an instance after " + str(poll_count) +" tries with " + str(poll_interval) + "s sleep in between")
        
    def failoverService(self, service_aquisition_callback, testcase_callback, **kwargs):
        ### Process Take down

        primary_service = service_aquisition_callback()
        secondary_service = self.tester.service_manager.wait_for_service(primary_service, state="DISABLED")
        self.tester.debug("Primary Service: " + primary_service.machine.hostname + " Secondary Service: " + secondary_service.machine.hostname)
        self.status("Failing over via service stop: " + str(primary_service.machine.hostname))
        primary_service.stop()
        
        if "clc" in primary_service.machine.components:
            self.tester.debug("Switching ec2 connection to host: " +  secondary_service.machine.hostname)
            self.tester.clc = secondary_service.machine
            self.tester.ec2.host = secondary_service.machine.hostname
        
        if "ws" in primary_service.machine.components:
            self.tester.debug("Switching walrus connection to host: " +  secondary_service.machine.hostname)
            self.tester.walrus = secondary_service.machine
            self.tester.s3.host = secondary_service.machine.hostname

        self.run_testcase(testcase_callback, **kwargs)

        after_failover = self.tester.service_manager.wait_for_service(primary_service, state="ENABLED")
        if primary_service.hostname is after_failover.hostname:
            self.fail("The enabled CLC was the same before and after the failover")     

        ### REMOVE DISABLED LOCK FILE FROM NON ACTIVE CLC AFTER 3.1
        #if not re.search("^3.1", self.version):
        #    primary_service.machine.sys("rm -rf " + self.tester.eucapath + "/var/lib/eucalyptus/db/data/disabled.lock")
        primary_service.start()
        
        try:
            self.servman.wait_for_service(primary_service, state ="DISABLED")
        except Exception, e:
            self.fail("The secondary service never went to disabled")
        
    
    
    def failoverReboot(self, service_aquisition_callback, testcase_callback, **kwargs):
        ### Reboot the current enabled component
        primary_service = service_aquisition_callback()
        secondary_service = self.tester.service_manager.wait_for_service(primary_service, state="DISABLED")
        self.tester.debug("Primary Service: " + primary_service.machine.hostname + " Secondary Service: " + secondary_service.machine.hostname)
        self.status("Failing over via reboot: " + str(primary_service.machine.hostname))
        primary_service.machine.reboot()    
        
        if "clc" in primary_service.machine.components:
            self.tester.debug("Switching ec2 connection to host: " +  secondary_service.machine.hostname)
            self.tester.clc = secondary_service.machine
            self.tester.ec2.host = secondary_service.machine.hostname
        
        if "ws" in primary_service.machine.components:
            self.tester.debug("Switching walrus connection to host: " +  secondary_service.machine.hostname)
            self.tester.walrus = secondary_service.machine
            self.tester.s3.host = secondary_service.machine.hostname

        after_failover =  self.tester.service_manager.wait_for_service(primary_service, state="ENABLED")

        self.run_testcase(testcase_callback, **kwargs)

        if primary_service.hostname is after_failover.hostname:
            self.fail("The enabled CLC was the same before and after the failover")     
             
        try:
            self.servman.wait_for_service(primary_service, state ="DISABLED")
        except Exception, e:
            self.fail("The secondary service never went to disabled")
            
    def failoverNetwork(self, service_aquisition_callback, testcase_callback, **kwargs):
        ### Reboot the current enabled component
        primary_service = service_aquisition_callback()
        secondary_service = self.tester.service_manager.wait_for_service(primary_service, state="DISABLED")
        self.tester.debug("Primary Service: " + primary_service.machine.hostname + " Secondary Service: " + secondary_service.machine.hostname)
        self.status("Failing over via network outage: " + str(primary_service.machine.hostname))

        interrupt_length = 800
        interrupt_start = int(time.time())
        primary_service.machine.interrupt_network(interrupt_length)
        
        if "clc" in primary_service.machine.components:
            self.tester.debug("Switching ec2 connection to host: " +  secondary_service.machine.hostname)
            self.tester.clc = secondary_service.machine
            self.tester.ec2.host = secondary_service.machine.hostname
        
        if "ws" in primary_service.machine.components:
            self.tester.debug("Switching walrus connection to host: " +  secondary_service.machine.hostname)
            self.tester.walrus = secondary_service.machine
            self.tester.s3.host = secondary_service.machine.hostname

        after_failover = self.servman.wait_for_service(primary_service, state ="ENABLED", timeout=interrupt_length)

        self.run_testcase(testcase_callback, **kwargs)

        testcase_finish = int(time.time()) - interrupt_start


        if primary_service.hostname is after_failover.hostname:
            self.fail("The enabled CLC was the same before and after the failover")

        outage_window_left = interrupt_length - testcase_finish
        if outage_window_left > 0:
            self.status("Sleeping wating for interfaces to come back up")
            self.tester.sleep(interrupt_length - testcase_finish)

        try:
            self.servman.wait_for_service(primary_service, state ="DISABLED")
        except Exception, e:
            self.fail("The secondary service never went to disabled")
    
    def post_run_checks(self):

        ## Refresh all SSH connections to machines
        for service in self.servman.get_all_services():
            service.machine.refresh_ssh()
        
        ## Ensure instance still has volume attached
        for instance in self.standing_reservation.instances:
            instance.ssh.refresh_connection()
            instance.assertFilePresent(self.device)
        
        key = self.tester.get_objects_by_prefix(self.standing_bucket_name, self.standing_key_name)
        
        self.servman.all_services_operational()
        
    def failoverCLC(self):
        self.failoverReboot(self.servman.get_enabled_clc, self.MetaData)
        self.post_run_checks()
        self.failoverService(self.servman.get_enabled_clc, self.MetaData)
        self.post_run_checks()
        self.failoverNetwork(self.servman.get_enabled_clc, self.MetaData)
        self.post_run_checks()
    
    def failoverWalrus(self):
        enabled_walrus = self.servman.get_enabled_walrus()
        try:
            enabled_walrus.machine.sys("ls " + self.tester.eucapath + "/var/lib/eucalyptus/bukkits/" + self.standing_bucket_name, code=0)
        except Exception, e:
            raise Exception("Unable to find bucket before failovers")
        self.failoverService(self.servman.get_enabled_walrus, self.test_bucket_key_list_delim_prefix)
        self.post_run_checks()
        self.failoverReboot(self.servman.get_enabled_walrus,self.test_bucket_key_list_delim_prefix)
        self.post_run_checks()
        self.failoverNetwork(self.servman.get_enabled_walrus, self.test_bucket_key_list_delim_prefix)
        self.post_run_checks()
    
    def failoverCC(self):
        zone = self.servman.partitions.keys()[0] 
        self.failoverService(self.servman.partitions[zone].get_enabled_cc, self.MetaData, zone=self.zone)
        self.post_run_checks()
        self.failoverReboot(self.servman.partitions[zone].get_enabled_cc, self.MetaData, zone=self.zone)
        self.post_run_checks()
        self.failoverNetwork(self.servman.partitions[zone].get_enabled_cc, self.MetaData, zone=self.zone)
        self.post_run_checks()
    
    def failoverSC(self):
        zone = self.servman.partitions.keys()[0]
        self.failoverService(self.servman.partitions[zone].get_enabled_sc, self.Reboot, zone=self.zone)
        self.post_run_checks()
        self.failoverReboot(self.servman.partitions[zone].get_enabled_sc, self.Reboot, zone=self.zone)
        self.post_run_checks()
        self.failoverNetwork(self.servman.partitions[zone].get_enabled_sc, self.Reboot, zone=self.zone)
        self.post_run_checks()
    
    def failoverVB(self):
        zone = self.servman.partitions.keys()[0]
        if len(self.servman.partitions[zone].vbs) > 1:
            self.failoverService(self.servman.partitions[zone].get_enabled_vb, self.Reboot, zone=self.zone)
            self.post_run_checks()
            self.failoverReboot(self.servman.partitions[zone].get_enabled_vb, self.Reboot, zone=self.zone)
            self.post_run_checks()
            self.failoverNetwork(self.servman.partitions[zone].get_enabled_vb, self.Reboot, zone=self.zone)
            self.post_run_checks()


if __name__ == "__main__":
    testcase = HAtests()
    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    list = testcase.args.tests or [ "failoverCLC", "failoverWalrus", "failoverCC", "failoverSC", "failoverVB"]
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )
        ### Run the EutesterUnitTest objects

    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)
   
